// ############################################################################################
//  Dialog.cc
//  A demo application for VortexGE
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include <VMain.h>


// Just for shortcut
typedef CBasicApplication<CEventHandler> MyApplicationBase;

// Specialized basic application class
class MyApplication : public MyApplicationBase {
  public:
    MyApplication();
    ~MyApplication() throw();
    virtual void EventKeyPress(EKeySymbol eKeySym);
    virtual void EventButtonPress(EBtnSymbol eButtonSym);
    virtual void EventButtonRelease(EBtnSymbol eButtonSym);
    virtual void EventMouseMove(int32_t n32XPos, int32_t n32YPos);
    virtual void EventUpdateFrame(bool bWindowIsActive);
    virtual void EventCloseWindow();

  private:
    bool          bInactive;
    EKeySymbol    eKey;
    CVideoStorage csVStrg;
    CPanelBar     csPBar;
    CHLine        csHLn1, csHLn2;
    CVLine        csVLn1, csVLn2;
    CFrame        csFra1, csFra2, csFra3, csFra4;
    CLabel        csLbl1, csLbl2, csLbl3, csLbl4;
    CButton       csBtn1, csBtn2, csBtn3, csBtn4, csBtnE;
    CCheckBox     csChk1, csChk2, csChk3, csChk4;
    CRadioButton  csRdo1, csRdo2, csRdo3, csRdo4;
    CRadioButton  csRdoA, csRdoB, csRdoC, csRdoD;
    CSpinButton   csSpn1, csSpn2, csSpn3, csSpn4, csSpn5, csSpn6;
    CListBox      csLst1, csLst2;
    CSLTextBox    csStb1, csStb2, csStb3;
    CControlList  csCList;

    // Helper functions
    void _initializeAllControls();
    void _redrawAllControls();

#ifndef __USE_UTF8_DIALOG_STRING__
    static bool _sltbCallback(ETBEvent eventCode, CVideoEnv &instVideoEnv,
                              CSLTextBox &slTextBox, string &editBuffer,
                              string *pasteBuffer, char &charCode, 
                              EKeySymbol keySym);
#else
    static bool _sltbCallback(ETBEvent eventCode, CVideoEnv &instVideoEnv,
                              CSLTextBox &slTextBox, CUString &editBuffer,
                              CUString *pasteBuffer, CUString::Utf32 &charCode,
                              EKeySymbol keySym);
#endif
};


// Main program
int main()
{
  MyApplication csMyApp;
  return(csMyApp.DoMainLoop());
}


// MyApplication implementation
MyApplication::MyApplication()
: MyApplicationBase("Demo : Simple Dialog Demo", NULL, 640, 480, 0, 0, EAIM_Disabled, E3DIM_Disabled),
  bInactive(false)
{
  // Initialize video storage
  csVStrg.Initialize(&iVideoEnv(), 1);

  // Initialize style
  TStyle tStyle;
  iVideoEnv().GetStyle(&tStyle);
  tStyle.xOutline         = iVideoEnv().CreateColor( 16,  16,  32);
  tStyle.xLightBorder     = iVideoEnv().CreateColor(164, 164, 255);
  tStyle.xDarkBorder      = iVideoEnv().CreateColor( 96,  96, 128);
  tStyle.xFace            = iVideoEnv().CreateColor(128, 128, 164);
  tStyle.xFaceLocked      = iVideoEnv().CreateColor(164, 140, 180);
  tStyle.xFaceSelect      = iVideoEnv().CreateColor(164, 128, 255);
  tStyle.xText            = iVideoEnv().CreateColor(  0,   0,  16);
  tStyle.xHotPress        = iVideoEnv().CreateColor( 96,  64, 164);
  tStyle.xIconBorder      = iVideoEnv().CreateColor( 32,  32,  96);
  tStyle.xIconBackground  = iVideoEnv().CreateColor(  8,   8,  16);
  tStyle.xIconInformation = iVideoEnv().CreateColor( 64,  64, 255);
  tStyle.xIconQuestion    = iVideoEnv().CreateColor( 64, 255,  64);
  tStyle.xIconWarning     = iVideoEnv().CreateColor(255, 255,  64);
  tStyle.xIconError       = iVideoEnv().CreateColor(255,  64,  64);
  tStyle.xFillStyle       = FILL_GRADIENT;
  iVideoEnv().SetStyle(&tStyle);

  // Initialize dialog
  _initializeAllControls();
  _redrawAllControls();
}

MyApplication::~MyApplication() throw()
{
}

void MyApplication::EventKeyPress(EKeySymbol eKeySym)
{
  if(eKeySym == VKS_ESC) EventCloseWindow();
}

void MyApplication::EventButtonPress(EBtnSymbol eButtonSym)
{
  if(eButtonSym == VBS_MB1) {
    /*
      You should write :
        VCE_ONMOUSEDOWN_START(<your video environment>);
          ...
          <your specific action, optional>
          ...
          VCE_ONMOUSEDOWN_DRAW(<your #1 control>);
          VCE_ONMOUSEDOWN_DRAW(<your #2 control>);
          VCE_ONMOUSEDOWN_DRAW(<your #3 control>);
          ...
        VCE_ONMOUSEDOWN_END();
    */
    VCE_ONMOUSEDOWN_START(iVideoEnv());
    {
      for(CControlList::const_iterator it = csCList.begin(); it != csCList.end(); ++it)
        VCE_ONMOUSEDOWN_DRAW(*it);
      /*
      VCE_ONMOUSEDOWN_DRAW(csBtn1); VCE_ONMOUSEDOWN_DRAW(csBtn2); VCE_ONMOUSEDOWN_DRAW(csBtn3); VCE_ONMOUSEDOWN_DRAW(csBtn4);
      VCE_ONMOUSEDOWN_DRAW(csBtnE);
      VCE_ONMOUSEDOWN_DRAW(csChk1); VCE_ONMOUSEDOWN_DRAW(csChk2); VCE_ONMOUSEDOWN_DRAW(csChk3); VCE_ONMOUSEDOWN_DRAW(csChk4);
      VCE_ONMOUSEDOWN_DRAW(csRdo1); VCE_ONMOUSEDOWN_DRAW(csRdo2); VCE_ONMOUSEDOWN_DRAW(csRdo3); VCE_ONMOUSEDOWN_DRAW(csRdo4);
      VCE_ONMOUSEDOWN_DRAW(csRdoA); VCE_ONMOUSEDOWN_DRAW(csRdoB); VCE_ONMOUSEDOWN_DRAW(csRdoC); VCE_ONMOUSEDOWN_DRAW(csRdoD);
      VCE_ONMOUSEDOWN_DRAW(csSpn1); VCE_ONMOUSEDOWN_DRAW(csSpn2); VCE_ONMOUSEDOWN_DRAW(csSpn3); VCE_ONMOUSEDOWN_DRAW(csSpn4);
      VCE_ONMOUSEDOWN_DRAW(csSpn5); VCE_ONMOUSEDOWN_DRAW(csSpn6);
      VCE_ONMOUSEDOWN_DRAW(csLst1); VCE_ONMOUSEDOWN_DRAW(csLst2);
      VCE_ONMOUSEDOWN_DRAW(csStb1); VCE_ONMOUSEDOWN_DRAW(csStb2); VCE_ONMOUSEDOWN_DRAW(csStb3);
      */
    }
    VCE_ONMOUSEDOWN_END();
  }
}

void MyApplication::EventButtonRelease(EBtnSymbol eButtonSym)
{
  if(eButtonSym == VBS_MB1) {
    /*
      You should write :
        VCE_ONMOUSEUP_START(<your video environment>);
          ...
          <your specific action, optional>
          ...
          <action set 1>
          <action set 2>
          <action set 3>
          ...
        VCE_ONMOUSEUP_END();
    */
    VCE_ONMOUSEUP_START(iVideoEnv());
    {

      /*
        For command buttons, you should write :
          VCE_ONMOUSEUP_ACTION_COMMANDBUTTON_BEGIN(<your command button>)
            ...
            <your specific action, optional>
            ...
          VCE_ONMOUSEUP_ACTION_END();
          ...
        or if you only need a single action, simply write :
          VCE_ONMOUSEUP_ACTION_COMMANDBUTTON_DEFAULT(<your command button>, <your action>)
          ...
      */
      VCE_ONMOUSEUP_ACTION_COMMANDBUTTON_BEGIN(csBtn1);
      {
        csVStrg.StoreCurrentView();
        iVideoEnv().MessageBox("Type 1", "OK-Cancel MessageBox", EMBI_Information, EMBB_OKCancel);
        csVStrg.RestoreCurrentView();
      }
      VCE_ONMOUSEUP_ACTION_END();
      VCE_ONMOUSEUP_ACTION_COMMANDBUTTON_BEGIN(csBtn2);
      {
        csVStrg.StoreCurrentView();
        iVideoEnv().MessageBox("Type 2", "Yes-No MessageBox", EMBI_Question, EMBB_YesNo);
        csVStrg.RestoreCurrentView();
      }
      VCE_ONMOUSEUP_ACTION_END();
      VCE_ONMOUSEUP_ACTION_COMMANDBUTTON_BEGIN(csBtn3);
      {
        csVStrg.StoreCurrentView();
        iVideoEnv().MessageBox("Type 3", "Yes-No-Cancel MessageBox", EMBI_Warning, EMBB_YesNoCancel);
        csVStrg.RestoreCurrentView();
      }
      VCE_ONMOUSEUP_ACTION_END();
      VCE_ONMOUSEUP_ACTION_COMMANDBUTTON_BEGIN(csBtn4);
      {
        csVStrg.StoreCurrentView();
        iVideoEnv().MessageBox("Type 4", "Abort-Retry-Ignore MessageBox", EMBI_Error, EMBB_AbortRetryIgnore);
        csVStrg.RestoreCurrentView();
      }
      VCE_ONMOUSEUP_ACTION_END();
      VCE_ONMOUSEUP_ACTION_COMMANDBUTTON_DEFAULT(csBtnE, EventCloseWindow());

      /*
        For check boxes, you may write :
          VCE_ONMOUSEUP_ACTION_CHECKBOX_BEGIN(<your check box>);
            ...
            <your specific action, optional>
            ...
          VCE_ONMOUSEUP_ACTION_END();
          ...
        or if you won't need any specific action, simply write :
          VCE_ONMOUSEUP_ACTION_CHECKBOX_DEFAULT(<your check box>)
          ...
      */
      VCE_ONMOUSEUP_ACTION_CHECKBOX_DEFAULT(csChk1);
      VCE_ONMOUSEUP_ACTION_CHECKBOX_DEFAULT(csChk2);
      VCE_ONMOUSEUP_ACTION_CHECKBOX_DEFAULT(csChk3);
      VCE_ONMOUSEUP_ACTION_CHECKBOX_DEFAULT(csChk4);

      /*
        For radio buttons, you should write :
          VCE_ONMOUSEUP_ACTION_RADIOBUTTON_GROUP(<radio 1>) &<radio 2>, &<radio 3>, ...
          VCE_ONMOUSEUP_ACTION_RADIOBUTTON_BEGIN();
            ...
            <your specific action, optional>
            ...
            VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(<radio 1>);
            VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(<radio 2>);
            VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(<radio 3>);
            ...
          VCE_ONMOUSEUP_ACTION_END();
          ...
      */
      VCE_ONMOUSEUP_ACTION_RADIOBUTTON_GROUP(csRdo1) &csRdo2, &csRdo3, &csRdo4
      VCE_ONMOUSEUP_ACTION_RADIOBUTTON_BEGIN();
      {
        VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(csRdo1); VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(csRdo2);
        VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(csRdo3); VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(csRdo4);
      }
      VCE_ONMOUSEUP_ACTION_END();
      VCE_ONMOUSEUP_ACTION_RADIOBUTTON_GROUP(csRdoA) &csRdoB, &csRdoC, &csRdoD
      VCE_ONMOUSEUP_ACTION_RADIOBUTTON_BEGIN();
      {
        VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(csRdoA); VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(csRdoB);
        VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(csRdoC); VCE_ONMOUSEUP_ACTION_RADIOBUTTON_DRAW(csRdoD);
      }
      VCE_ONMOUSEUP_ACTION_END();

      /*
        For spin buttons, you may write :
          VCE_ONMOUSEUP_ACTION_SPINBUTTON_BEGIN(<your spin button>);
            ...
            <your specific action, optional>
            ...
          VCE_ONMOUSEUP_ACTION_END();
          ...
        or if you won't need any specific action, simply write :
          VCE_ONMOUSEUP_ACTION_SPINBUTTON_DEFAULT(<your spin button>)
          ...
      */
      VCE_ONMOUSEUP_ACTION_SPINBUTTON_DEFAULT(csSpn1);
      VCE_ONMOUSEUP_ACTION_SPINBUTTON_DEFAULT(csSpn2);
      VCE_ONMOUSEUP_ACTION_SPINBUTTON_DEFAULT(csSpn3);
      VCE_ONMOUSEUP_ACTION_SPINBUTTON_DEFAULT(csSpn4);
      VCE_ONMOUSEUP_ACTION_SPINBUTTON_DEFAULT(csSpn5);
      VCE_ONMOUSEUP_ACTION_SPINBUTTON_DEFAULT(csSpn6);

      /*
        For list boxes, you may write :
          VCE_ONMOUSEUP_ACTION_LISTBOX_BEGIN(<your list box>);
            ...
            <your specific action, optional>
            ...
          VCE_ONMOUSEUP_ACTION_END();
          ...
        or if you won't need any specific action, simply write :
          VCE_ONMOUSEUP_ACTION_LISTBOX_DEFAULT(<your list box>)
          ...
      */
      VCE_ONMOUSEUP_ACTION_LISTBOX_DEFAULT(csLst1);
      VCE_ONMOUSEUP_ACTION_LISTBOX_DEFAULT(csLst2);

      /*
        For single line text boxes, you may write :
          VCE_ONMOUSEUP_ACTION_SLTEXTBOX_BEGIN(<your single line text box>);
            ...
            if(ACCEPTED) {
              ...
              <your specific action if the changes accepted, optional>
              ...
            }
            else {
              ...
              <your specific action if the changes rejected, optional>
              ...
            }
            ...
          VCE_ONMOUSEUP_ACTION_END();
          ...
        or if you won't need any specific action, simply write :
          VCE_ONMOUSEUP_ACTION_SLTEXTBOX_DEFAULT(<your single line text box>)
          ...
        
        You may also use the extended macro to support event propagation :
          ...
          VCE_ONMOUSEUP_ACTION_SLTEXTBOX_BEGIN_EX(<your single line text box>, this);
          ...
        or :
          ...
          VCE_ONMOUSEUP_ACTION_SLTEXTBOX_DEFAULTEX(<your single line text box>, this)
          ...
      */
      VCE_ONMOUSEUP_ACTION_SLTEXTBOX_DEFAULT_EX(csStb1, this);
      VCE_ONMOUSEUP_ACTION_SLTEXTBOX_DEFAULT_EX(csStb2, this);
      VCE_ONMOUSEUP_ACTION_SLTEXTBOX_DEFAULT_EX(csStb3, this);
    }
    VCE_ONMOUSEUP_END();
  }
}

void MyApplication::EventMouseMove(int32_t n32XPos, int32_t n32YPos)
{
  UNUSED_PARAMS_2(n32XPos, n32YPos);
  /*
    You should write :
      VCE_ONMOUSEMOVE_START(<your video environment>, VCE_BUTTON?_STATE);
        ...
        <your specific action, optional>
        ...
        VCE_ONMOUSEMOVE_DRAW(<your #1 control>);
        VCE_ONMOUSEMOVE_DRAW(<your #2 control>);
        VCE_ONMOUSEMOVE_DRAW(<your #3 control>);
        ...
      VCE_ONMOUSEMOVE_END();
  */
  VCE_ONMOUSEMOVE_START(iVideoEnv(), VCE_MOUSE_BUTTON_1);
  {
    for(CControlList::const_iterator it = csCList.begin(); it != csCList.end(); ++it)
      VCE_ONMOUSEMOVE_DRAW(*it);
    /*
    VCE_ONMOUSEMOVE_DRAW(csBtn1); VCE_ONMOUSEMOVE_DRAW(csBtn2); VCE_ONMOUSEMOVE_DRAW(csBtn3); VCE_ONMOUSEMOVE_DRAW(csBtn4);
    VCE_ONMOUSEMOVE_DRAW(csBtnE);
    VCE_ONMOUSEMOVE_DRAW(csChk1); VCE_ONMOUSEMOVE_DRAW(csChk2); VCE_ONMOUSEMOVE_DRAW(csChk3); VCE_ONMOUSEMOVE_DRAW(csChk4);
    VCE_ONMOUSEMOVE_DRAW(csRdo1); VCE_ONMOUSEMOVE_DRAW(csRdo2); VCE_ONMOUSEMOVE_DRAW(csRdo3); VCE_ONMOUSEMOVE_DRAW(csRdo4);
    VCE_ONMOUSEMOVE_DRAW(csRdoA); VCE_ONMOUSEMOVE_DRAW(csRdoB); VCE_ONMOUSEMOVE_DRAW(csRdoC); VCE_ONMOUSEMOVE_DRAW(csRdoD);
    VCE_ONMOUSEMOVE_DRAW(csSpn1); VCE_ONMOUSEMOVE_DRAW(csSpn2); VCE_ONMOUSEMOVE_DRAW(csSpn3); VCE_ONMOUSEMOVE_DRAW(csSpn4);
    VCE_ONMOUSEMOVE_DRAW(csSpn5); VCE_ONMOUSEMOVE_DRAW(csSpn6);
    VCE_ONMOUSEMOVE_DRAW(csLst1); VCE_ONMOUSEMOVE_DRAW(csLst2);
    VCE_ONMOUSEMOVE_DRAW(csStb1); VCE_ONMOUSEMOVE_DRAW(csStb2); VCE_ONMOUSEMOVE_DRAW(csStb3); 
    */
  }
  VCE_ONMOUSEMOVE_END();
}

void MyApplication::EventUpdateFrame(bool bWindowIsActive)
{
  if(!bWindowIsActive) {
    if(!bInactive) {
      bInactive = true;
      csVStrg.StoreCurrentView();
    }
    iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
    iVideoEnv().UseDefaultBigFont();
    iVideoEnv().DrawText(0, 150, "Inactive State", ETA_CenterScreen);
    iVideoEnv().UseDefaultSmallFont();
    iVideoEnv().UpdateDisplay();
  }
  else {
    if(bInactive) {
      bInactive = false;
      csVStrg.RestoreCurrentView();
      iVideoEnv().UpdateDisplay();
    }
    usleep(4000);
  }
}

void MyApplication::EventCloseWindow()
{
  iVideoEnv().ApplyBlackScreen();
  iVideoEnv().MessageBox("End Program", "Click \"OK\" to exit ...");
  MyApplicationBase::EventCloseWindow();
}

void MyApplication::_initializeAllControls()
{
  csPBar.InitControl(iVideoEnv(), NULL, 0, 0, 640, 480, "A Simple Dialog");
  csCList.add(&csPBar);

  csHLn1.InitControl(iVideoEnv(), &csPBar,   5, 425, 630, EBS_Raised);
  csHLn2.InitControl(iVideoEnv(), &csPBar,   5, 430, 630, EBS_Sunken);
  csVLn1.InitControl(iVideoEnv(), &csPBar, 505,  35, 385, EBS_Raised);
  csVLn2.InitControl(iVideoEnv(), &csPBar, 510,  35, 385, EBS_Sunken);
  csCList.add(&csHLn1); csCList.add(&csHLn2);
  csCList.add(&csVLn1); csCList.add(&csVLn2);

  csFra1.InitControl(iVideoEnv(), &csPBar, 15,  45, 240, 50,   "Raised Frame", EHTA_Left,   EBS_Raised);
  csFra2.InitControl(iVideoEnv(), &csPBar, 15, 110, 240, 60,   "Sunken Frame", EHTA_Left,   EBS_Sunken);
  csFra3.InitControl(iVideoEnv(), &csPBar, 15, 185, 240, 60, "Center Aligned", EHTA_Center, EBS_Raised);
  csFra4.InitControl(iVideoEnv(), &csPBar, 15, 260, 240, 60,  "Right Aligned", EHTA_Right,  EBS_Sunken);
  csCList.add(&csFra1); csCList.add(&csFra2);
  csCList.add(&csFra3); csCList.add(&csFra4);

  csLbl1.InitControl(iVideoEnv(), &csFra1, 120, 30,     "ELCOTopLeft", EHTA_Left,  EVTA_Top);
  csLbl2.InitControl(iVideoEnv(), &csFra1, 120, 30,    "ELCOTopRight", EHTA_Right, EVTA_Top);
  csLbl3.InitControl(iVideoEnv(), &csFra1, 120, 30,  "ELCOBottomLeft", EHTA_Left,  EVTA_Bottom);
  csLbl4.InitControl(iVideoEnv(), &csFra1, 120, 30, "ELCOBottomRight", EHTA_Right, EVTA_Bottom);
  csCList.add(&csLbl1); csCList.add(&csLbl2);
  csCList.add(&csLbl3); csCList.add(&csLbl4);

  csBtn1.InitControl(iVideoEnv(), &csPBar, 525,  40, 100, 30,   "Type 1", false, false);
  csBtn2.InitControl(iVideoEnv(), &csPBar, 525,  80, 100, 30,   "Type 2", false,  true);
  csBtn3.InitControl(iVideoEnv(), &csPBar, 525, 120, 100, 30,   "Type 3",  true, false);
  csBtn4.InitControl(iVideoEnv(), &csPBar, 525, 160, 100, 30,   "Type 4",  true,  true);
  csBtnE.InitControl(iVideoEnv(), &csPBar,  10, 440, 620, 30, _M("Exit"),  true, false);
  csCList.add(&csBtn1); csCList.add(&csBtn2);
  csCList.add(&csBtn3); csCList.add(&csBtn4);
  csCList.add(&csBtnE);

  csChk1.InitControl(iVideoEnv(), &csFra2,  10, 15, "Check Box 1", false);
  csChk2.InitControl(iVideoEnv(), &csFra2,  10, 35, "Check Box 2", false);
  csChk3.InitControl(iVideoEnv(), &csFra2, 228, 15, "Check Box 3",  true);
  csChk4.InitControl(iVideoEnv(), &csFra2, 228, 35, "Check Box 4",  true);
  csCList.add(&csChk1); csCList.add(&csChk2);
  csCList.add(&csChk3); csCList.add(&csChk4);

  csRdo1.InitControl(iVideoEnv(), &csFra3,  10, 15, "Radio Button 1", false);
  csRdo2.InitControl(iVideoEnv(), &csFra3,  10, 35, "Radio Button 2", false);
  csRdo3.InitControl(iVideoEnv(), &csFra3, 228, 15, "Radio Button 3",  true);
  csRdo4.InitControl(iVideoEnv(), &csFra3, 228, 35, "Radio Button 4",  true);
  csRdo1.SetValue(true);
  csRdoA.InitControl(iVideoEnv(), &csFra4,  10, 15, "Radio Button A", false);
  csRdoB.InitControl(iVideoEnv(), &csFra4,  10, 35, "Radio Button B", false);
  csRdoC.InitControl(iVideoEnv(), &csFra4, 228, 15, "Radio Button C",  true);
  csRdoD.InitControl(iVideoEnv(), &csFra4, 228, 35, "Radio Button D",  true);
  csRdoA.SetValue(true);
  csCList.add(&csRdo1); csCList.add(&csRdo2);
  csCList.add(&csRdo3); csCList.add(&csRdo4);
  csCList.add(&csRdoA); csCList.add(&csRdoB);
  csCList.add(&csRdoC); csCList.add(&csRdoD);

  csSpn1.InitControl(iVideoEnv(), &csPBar, 265, 45, 70, 40, ESBS_VLeft,  EHTA_Left,   ESBV_Integer,  true);
  csSpn3.InitControl(iVideoEnv(), &csPBar, 345, 45, 70, 40, ESBS_VLeft,  EHTA_Center, ESBV_Integer, false);
  csSpn5.InitControl(iVideoEnv(), &csPBar, 425, 45, 70, 40, ESBS_VLeft,  EHTA_Right,  ESBV_Integer,  true);
  csSpn2.InitControl(iVideoEnv(), &csPBar, 265, 95, 70, 40, ESBS_VRight, EHTA_Left,   ESBV_Float1,   true);
  csSpn4.InitControl(iVideoEnv(), &csPBar, 345, 95, 70, 40, ESBS_VRight, EHTA_Center, ESBV_Float1,  false);
  csSpn6.InitControl(iVideoEnv(), &csPBar, 425, 95, 70, 40, ESBS_VRight, EHTA_Right,  ESBV_Float1,   true);
  csCList.add(&csSpn1); csCList.add(&csSpn2);
  csCList.add(&csSpn3); csCList.add(&csSpn4);
  csCList.add(&csSpn5); csCList.add(&csSpn6);

  csLst1.InitControl(iVideoEnv(), &csPBar, 265, 145, 230, 130);
  csLst2.InitControl(iVideoEnv(), &csPBar, 265, 285, 230, 130);
  csLst1.AddItems(3, "One", "Two", "Three");
  csLst2.AddItems(26, "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M",
                      "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z");
  csCList.add(&csLst1); csCList.add(&csLst2);

  csStb1.InitControl(iVideoEnv(), &csPBar, 15, 330, 240, 23, ETBT_TextNormal);
  csStb2.InitControl(iVideoEnv(), &csPBar, 15, 361, 240, 23, ETBT_Hexadecimal, false, MyApplication::_sltbCallback);
  csStb3.InitControl(iVideoEnv(), &csPBar, 15, 392, 240, 23, ETBT_Float, false, MyApplication::_sltbCallback);
  //csStb1.SetText("ぁあぃいぅうぇえぉお");
  csStb1.SetText("Ay Ay Ay");
  csStb2.SetText("F0A3");
  csStb3.SetText("-10.3e5");
  csCList.add(&csStb1); csCList.add(&csStb2);
  csCList.add(&csStb3);
}

void MyApplication::_redrawAllControls()
{
  iVideoEnv().UseDefaultSmallFont();
  for(CControlList::const_iterator it = csCList.begin(); it != csCList.end(); ++it)
    it->DrawControl();
  iVideoEnv().UpdateDisplay();
}

#ifndef __USE_UTF8_DIALOG_STRING__
bool MyApplication::_sltbCallback(ETBEvent eventCode, CVideoEnv &instVideoEnv,
                                  CSLTextBox &slTextBox, string &editBuffer,
                                  string *pasteBuffer, char &charCode,
                                  EKeySymbol keySym)
#else
bool MyApplication::_sltbCallback(ETBEvent eventCode, CVideoEnv &instVideoEnv,
                                  CSLTextBox &slTextBox, CUString &editBuffer,
                                  CUString *pasteBuffer, CUString::Utf32 &charCode,
                                  EKeySymbol keySym)
#endif
{
  UNUSED_PARAMS_6(instVideoEnv, slTextBox, editBuffer, pasteBuffer, charCode, keySym);

  if(eventCode == ETBE_Paste) return(false);
  return(true);
}
