// ############################################################################################
//  RotatingBox.cc
//  A demo application for VortexGE
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

//#define __HIGH_RESOLUTION__

// Common headers for demo application
#include "DemoCommon.h"


// Just for shortcut
typedef CBasicApplication<CEventHandler> MyApplicationBase;

// Specialized basic application class
class MyApplication : public MyApplicationBase {
  public:
    MyApplication();
    virtual void EventKeyPress(EKeySymbol eKeySym);
    virtual void EventUpdateFrame(bool bWindowIsActive);
    virtual void EventCloseWindow();

  private:
    CPanel3DBuffer csGround, csBaseBox, csRotBox;
    CMatrix3D      csMatCamera, csMatBaseBox, csMatRotBox1, csMatRotBox2;
    CMatrix3D      csMatTm0, csMatTm1, csMatTm2;
    CImageBuilder  csImgBuild;
    CImage         csImgBack;
    CTexture       csTGround, csTBox;
    float          fXDir;
    int16_t        n16XCnt;
};


// Main program
int main()
{
  MyApplication csMyApp;
  return(csMyApp.DoMainLoop());
}


// MyApplication implementation
MyApplication::MyApplication()
#ifdef __HIGH_RESOLUTION__
: MyApplicationBase("Demo : Rotating Box", NULL, 640, 480, 640, 480, EAIM_Disabled),
#else
: MyApplicationBase("Demo : Rotating Box", NULL, 640, 480, 320, 240, EAIM_Disabled),
#endif
  fXDir(0.2f), n16XCnt(200)
{
  // Tell the user to be patient
  ShowInitSystemPleaseWaitMessage(iVideoEnv());

  // Load background image
  csImgBuild.LoadBMP("../media/image/back1.bmp", true);
  csImgBuild.ResizeToFitBackground();
  csImgBuild.AdjustBrightness(0.3, 0);
  csImgBuild.SmearEffect(5);
  csImgBuild.TransferToImage(csImgBack);

  // Load texture image 1
  try {
    csImgBuild.LoadJPG("../media/image/tex1.jpg", true);
    csImgBuild.Resize(256, 256);
    csImgBuild.SmearEffect(2);
  }
  catch(...) {
    csImgBuild.CreateNoiseImage(256, 256, 4, 22, 4, 2, 8, 2);
  }
  csImgBuild.TransferToTexture(csTGround);

  // Load texture image 2
  try {
    csImgBuild.LoadJPG("../media/image/tex2.jpg", true);
    csImgBuild.Resize(64, 64);

  }
  catch(...) {
    csImgBuild.CreateNoiseImage(64, 64, 18, 18, 20, 2, 2, 2);
    csImgBuild.OilPaintingEffect(1);
  }
  csImgBuild.TransferToTexture(csTBox);

  // Free image
  csImgBuild.FreeImage();

  // Create scene
  i3DEnv().SetFogState(8, 8, 12, 13);
  i3DEnv().SetDefaultOmniLightPosition(40, 200, 150);
  i3DEnv().SetDefaultOmniLightColor(31, 31, 16);
  i3DEnv().SetZClipping(150, 500);

  float    afVG[]    = { -100,-10,100,   -100,-10,200,   100,-10,200,   100,-10,100,
                         -100,  0,100,   -100,  0,200,   100,  0,200,   100,  0,100 };
  uint32_t aun32PG[] = { 3,2,1,0,   4,5,6,7,
                         0,4,7,3,   3,7,6,2,    2,6,5,1,   1,5,4,0 };
  csGround.InitFromMemory(afVG, 8, aun32PG, 6);
  csGround.SetTexture(&csTGround);
  csGround.CalcDefTexCoords();

  float    afVB[]    = { 0, 0,-20,   -20, 0,0,   0, 0,20,   20, 0,0,
                         0,35,-20,   -20,35,0,   0,35,20,   20,35,0 };
  uint32_t aun32PB[] = { 3,2,1,0,   4,5,6,7,
                         0,4,7,3,   3,7,6,2,   2,6,5,1,   1,5,4,0 };
  csBaseBox.InitFromMemory(afVB, 8, aun32PB, 6);
  csBaseBox.SetTexture(&csTBox);
  csBaseBox.CalcDefTexCoords();

  float    afVR[]    = { 0, 0,-10,   0, 0,10,   60, 0,10,   60, 0,-10,
                         0,10,-10,   0,10,10,   60,10,10,   60,10,-10 };
  uint32_t aun32PR[] = { 3,2,1,0,   4,5,6,7,
                         0,4,7,3,   3,7,6,2,   2,6,5, 1,   1,5,4,0 };
  csRotBox.InitFromMemory(afVR, 8, aun32PR, 6);
  csRotBox.SetTexture(&csTBox);
  csRotBox.CalcDefTexCoords();

  // Initialize matrix
  csMatCamera.SetToIdentityMatrix();
  csMatCamera.Translate(0, -200, 100);
  csMatCamera.Rotate(-400, 0, 0);

  csMatBaseBox.SetToIdentityMatrix();
  csMatBaseBox.Translate(0, -5, 150);
  csMatRotBox1.SetToIdentityMatrix();
  csMatRotBox2.SetToIdentityMatrix();
  csMatRotBox2.Translate(-30, 0, 0);
  csMatRotBox2.Scale(0.5, 0.5, 0.5);
}

void MyApplication::EventKeyPress(EKeySymbol eKeySym)
{
  if(eKeySym == VKS_ESC) EventCloseWindow();
}

void MyApplication::EventUpdateFrame(bool bWindowIsActive)
{
  // Update frame (if the application is not active)
  if(!bWindowIsActive) {
    iVideoEnv().PutRenderImageOnDisplay();
    iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
    iVideoEnv().UseDefaultBigFont();
    iVideoEnv().DrawText(0, 150, "Inactive State", ETA_CenterScreen);
    iVideoEnv().UseDefaultSmallFont();
    iVideoEnv().UpdateDisplay();
    return;
  }

  // Update frame
  csMatBaseBox.Translate(fXDir , 0, 0);
  csMatRotBox1.Rotate(0, 10, 0);
  csMatRotBox2.Rotate(0, -25, 0);
  n16XCnt++;
  if(n16XCnt > 400) {
    fXDir = -fXDir;
    n16XCnt = 0 ;
  }

  // Clear scene
  i3DEnv().ClearRenderBuffer(csImgBack);
  i3DEnv().ClearZBuffer();

  // Render the ground
  csGround.FogTGRender(csMatCamera);
  // Render the base box
  csMatTm0.SetFromMatrix(csMatBaseBox);
  csMatTm0.MultMatrixCCN(csMatCamera);
  csBaseBox.FogTGRender(csMatTm0);
  // Render the 1st rotating box
  csMatTm1.SetFromMatrix(csMatRotBox1);
  csMatTm1.Translate(0, 20, 0);
  csMatTm1.MultMatrixCCN(csMatTm0);
  csRotBox.FogTGRender(csMatTm1);
  // Render the 2nd rotating box
  csMatTm2.SetFromMatrix(csMatRotBox2);
  csMatTm2.Translate(50, 10, 0);
  csMatTm2.MultMatrixCCN(csMatTm1);
  csRotBox.FogTGRender(csMatTm2);

  // Update display
  iVideoEnv().PutRenderImageOnDisplay();
  iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
  iVideoEnv().DrawText(10, 20, "Press escape to exit ...");
  iVideoEnv().DrawText(10, 468, "The incorrect lighting is corrected in \"Rotating Box 2\" (RotatingBox2.cc).");
  iVideoEnv().UpdateDisplay();
}

void MyApplication::EventCloseWindow()
{
  iVideoEnv().MessageBox("End Program", "Click \"OK\" to exit ...");
  MyApplicationBase::EventCloseWindow();
}
