/** \file SkyObjectFractals.h */
// ############################################################################################
//  SkyObjectFractals.h
//  Definition & implementation of various CIndefiniteAreaFractal classes
//  specialized to imitate sky objects
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  Some of the algorithms used in this file were written based on the book :
//    Fractal Vision : Put Fractals to Work for You
//    Dick Oliver
//    Sams Publishing
//    1992
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _2D_MANIPULATION___FRACTAL___SKY_OBJECT_FRACTALS_H_
#define _2D_MANIPULATION___FRACTAL___SKY_OBJECT_FRACTALS_H_

// VortexGE headers
#include "IndefiniteAreaFractals.h"


namespace vge {

  //!
  //! \brief Fractal that will try to imitate cirrus cloud
  //!
  //! \api experiment
  class VGE_EXPORT FractalCirrus : public CSeedReplicatorFractal {
    public:
      FractalCirrus();

    protected:
      // Parameter-control functions
      virtual uint32_t CosmosStartLevelThreshold() const { return(4); }
      virtual bool     CosmosUpdateColor(uint32_t un32MaxLevel, uint32_t un32CurLevel, uint32_t un32TotPart, uint32_t un32CurPart, uint32_t un32TotRelPart, uint32_t un32CurRelPart);
      virtual uint32_t ChaosStartLevelThreshold()  const { return(5); }
      virtual uint32_t ChaosEndLevelThreshold()    const { return(5); }
      virtual bool     ChaosUpdateColor(uint32_t un32MaxLevel, uint32_t un32CurLevel, uint32_t un32TotRelPart, uint32_t un32CurRelPart);
      virtual uint8_t  ChaosFactor()               const { return(3); }
  };


  //!
  //! \brief Fractal that will try to imitate a stargate-like appearance
  //!
  //! \api experiment
  class VGE_EXPORT FractalStargate : public CSeedReplicatorFractal {
    public:
      FractalStargate();

    protected:
      // Parameter-control functions
      virtual uint32_t CosmosStartLevelThreshold() const { return(1); }
      virtual bool     CosmosUpdateColor(uint32_t un32MaxLevel, uint32_t un32CurLevel, uint32_t un32TotPart, uint32_t un32CurPart, uint32_t un32TotRelPart, uint32_t un32CurRelPart);
      virtual uint32_t ChaosStartLevelThreshold()  const { return(1); }
      virtual uint32_t ChaosEndLevelThreshold()    const { return(3); }
      virtual bool     ChaosUpdateColor(uint32_t un32MaxLevel, uint32_t un32CurLevel, uint32_t un32TotRelPart, uint32_t un32CurRelPart);
      virtual uint8_t  ChaosFactor()               const { return(5); }
  };

} // namespace vge
#endif
