// ############################################################################################
//  AudioEnvironment.cc
//  Implementation of static functions of the CAudioEnv interface class and
//  common mixing functions
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGEs headers
#include "../System/SystemUtilities.h"
#include "AudioEnvironment.h"
#include "SoundBuffer.h"
#include "PCM_ALSA/AALSAEnvironment.h"
#include "PCM_Dummy/ADMYEnvironment.h"
#include "PCM_OSS/AOSSEnvironment.h"


namespace vge {

//
// Audio environment interface class
//
CAudioEnv::CAudioEnv()
{
}

CAudioEnv::~CAudioEnv() throw()
{
}

CAudioEnv& CAudioEnv::AutoDetect()
{
  // Get the user preference
  char *pcEnv = getenv("VGE_AUDIO_SYSTEM");

  // Try ALSA
#ifdef __USE_ALSA_AUDIO__
  if(VMatchEVar(pcEnv,"alsa") != EMEV_Mismatch) {
    CAlsaAudioEnv &csALSA = CAlsaAudioEnv::Instance();
    try {
      csALSA.QueryDeviceName(NULL);
      return(csALSA);
    }
    catch(...) {}
  }
#endif

  // Try OSS
#ifdef __USE_OSS_AUDIO__
  if(VMatchEVar(pcEnv,"oss") != EMEV_Mismatch) {
    COssAudioEnv &csOSS = COssAudioEnv::Instance();
    try {
      csOSS.QueryDeviceName(NULL);
      return(csOSS);
    }
    catch(...) {}
  }
#endif

  // Check for invalid values
  if((VMatchEVar(pcEnv, "dummy") == EMEV_Mismatch) &&
     (VMatchEVar(pcEnv,  "null") == EMEV_Mismatch))
    VWarning(_NFL_, "Invalid value in environment variable \"VGE_AUDIO_SYSTEM\"\nThe Dummy (NULL) audio system will be used");

  // Return dummy
  return(CDummyAudioEnv::Instance());
}


//
// Audio sound-buffer interface class
//
CSoundBuffer::CSoundBuffer()
{ 
  VCLASS_CAN_ABORT_ON_DESTRUCTION_CTOR_BEGIN("CSoundBuffer");
}

CSoundBuffer::~CSoundBuffer() throw()
{
}

} // namespace vge
