// ############################################################################################
//  MidiPlayer.cc
//  Implementation of static functions of the CMidiPlayer interface class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGEs headers
#include "../Base/MessageDisplayer.h"
#include "../System/SystemUtilities.h"
#include "MID_Dummy/MDMYPlayer.h"
#include "MID_libTiMidity/MTIMPlayer.h"


namespace vge {

//
// The generic MIDI player class
//
CMidiPlayer::~CMidiPlayer() throw()
{
}

CMidiPlayer& CMidiPlayer::AutoDetect()
{
  // Get the user preference
  char *pcEnv = getenv("VGE_MIDI_SYSTEM");

  // Try libTiMidity
#ifdef __USE_TIM_MIDI__
  if((VMatchEVar(pcEnv,    "timidity") != EMEV_Mismatch) ||
     (VMatchEVar(pcEnv, "libtimidity") != EMEV_Mismatch)) {
    CTiMidityPlayer &csTiMidity = CTiMidityPlayer::Instance();
    try {
      csTiMidity.InitializeDevice(NULL);
      return(csTiMidity);
    }
    catch(...) {}
  }
#endif

  // Check for invalid values
  if((VMatchEVar(pcEnv, "dummy") == EMEV_Mismatch) &&
     (VMatchEVar(pcEnv,  "null") == EMEV_Mismatch))
    VWarning(_NFL_, "Invalid value in environment variable \"VGE_MIDI_SYSTEM\"\nThe Dummy (NULL) MIDI system will be used");

  // Return dummy
  return(CDummyMidiPlayer::Instance());
}

} // namespace vge
