// ############################################################################################
//  NonDummyAudioEnvBaseClass.cc
//  Implementation of CAudioEnvCImp class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "NonDummyAudioEnvBaseClass.h"
#include "NonDummySoundBufferBaseClass.h"


// For experiment, just leave it as it
#define __USE_CMOV__


namespace vge {

//
// Used for swapping values
//
struct _TSwapValue
{
  int8_t n8Val1;
  int8_t n8Val2;
};

#define _SWAP_STORE(DST, SRC)                                                                    \
  (reinterpret_cast<_TSwapValue*>(DST))->n8Val1 = (reinterpret_cast<_TSwapValue*>(SRC))->n8Val2; \
  (reinterpret_cast<_TSwapValue*>(DST))->n8Val2 = (reinterpret_cast<_TSwapValue*>(SRC))->n8Val1; \
  ++DST


//
// Non-dummy audio environment common class
//
CAudioEnvCImp *CAudioEnvCImp::_instAudioEnv = NULL;

CAudioEnvCImp::CAudioEnvCImp()
: _tCD(NULL), _pcsMidiPlayer(NULL), _pfnDoSoundMixing(NULL)
{
}

CAudioEnvCImp::~CAudioEnvCImp() throw()
{
}


//
// Normal sound mixing functions
//
void CAudioEnvCImp::_DSPMixerU8m() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  uint8_t *pun8Buff = reinterpret_cast<uint8_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int8_t   n8Data;
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t n32Sum = 0;
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(&n8Data)
          );
          n32Sum += n8Data;
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
    register uint8_t tmp = 0;
    if(n32Sum >= -128) tmp = uint8_t(n32Sum) ^ 0x80;
    if(n32Sum >   127) tmp = 255;
    *pun8Buff++ = tmp;
#else
         if(n32Sum < -128) *pun8Buff++ = 0;
    else if(n32Sum >  127) *pun8Buff++ = 255;
    else                   *pun8Buff++ = uint8_t(n32Sum) ^ 0x80;
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_DSPMixerU8s() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  uint8_t *pun8Buff = reinterpret_cast<uint8_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int8_t   an8Data[2];
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t an32Sum[2] = {0, 0};
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(an8Data)
          );
          an32Sum[0] += an8Data[0];
          an32Sum[1] += an8Data[1];
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
    register uint8_t tmp0 = 0;
    if(an32Sum[0] >= -128) tmp0 = uint8_t(an32Sum[0]) ^ 0x80;
    if(an32Sum[0] >   127) tmp0 = 255;
    *pun8Buff++ = tmp0;
    register uint8_t tmp1 = 0;
    if(an32Sum[1] >= -128) tmp1 = uint8_t(an32Sum[1]) ^ 0x80;
    if(an32Sum[1] >   127) tmp1 = 255;
    *pun8Buff++ = tmp1;
#else
         if(an32Sum[0] < -128) *pun8Buff++ = 0;
    else if(an32Sum[0] >  127) *pun8Buff++ = 255;
    else                       *pun8Buff++ = uint8_t(an32Sum[0]) ^ 0x80;
         if(an32Sum[1] < -128) *pun8Buff++ = 0;
    else if(an32Sum[1] >  127) *pun8Buff++ = 255;
    else                       *pun8Buff++ = uint8_t(an32Sum[1]) ^ 0x80;
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_DSPMixerS8m() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  int8_t *pn8Buff = reinterpret_cast<int8_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int8_t  n8Data;
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t n32Sum = 0;
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(&n8Data)
          );
          n32Sum += n8Data;
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
    register int8_t tmp = -128;
    if(n32Sum >= -128) tmp = int8_t(n32Sum);
    if(n32Sum >   127) tmp = 127;
    *pn8Buff++ = tmp;
#else
         if(n32Sum < -128) *pn8Buff++ = -128;
    else if(n32Sum >  127) *pn8Buff++ =  127;
    else                   *pn8Buff++ = int8_t(n32Sum);
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_DSPMixerS8s() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  int8_t *pn8Buff = reinterpret_cast<int8_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int8_t  an8Data[2];
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t an32Sum[2] = {0, 0};
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(an8Data)
          );
          an32Sum[0] += an8Data[0];
          an32Sum[1] += an8Data[1];
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
    register int8_t tmp0 = -128;
    if(an32Sum[0] >= -128) tmp0 = int8_t(an32Sum[0]);
    if(an32Sum[0] >   127) tmp0 = 127;
    *pn8Buff++ = tmp0;
    register int8_t tmp1 = -128;
    if(an32Sum[1] >= -128) tmp1 = int8_t(an32Sum[1]);
    if(an32Sum[1] >   127) tmp1 = 127;
    *pn8Buff++ = tmp1;
#else
         if(an32Sum[0] < -128) *pn8Buff++ = -128;
    else if(an32Sum[0] >  127) *pn8Buff++ =  127;
    else                       *pn8Buff++ = int8_t(an32Sum[0]);
         if(an32Sum[1] < -128) *pn8Buff++ = -128;
    else if(an32Sum[1] >  127) *pn8Buff++ =  127;
    else                       *pn8Buff++ = int8_t(an32Sum[1]);
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_DSPMixerU16m() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  uint16_t *pun16Buff = reinterpret_cast<uint16_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int16_t   n16Data;
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t n32Sum = 0;
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(&n16Data)
          );
          n32Sum += n16Data;
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
    register uint16_t tmp = 0;
    if(n32Sum >= -32768) tmp = uint16_t(n32Sum) ^ 0x8000;
    if(n32Sum >   32767) tmp = 65535;
    *pun16Buff++ = tmp;
#else
         if(n32Sum < -32768) *pun16Buff++ = 0;
    else if(n32Sum >  32767) *pun16Buff++ = 65535;
    else                     *pun16Buff++ = uint16_t(n32Sum) ^ 0x8000;
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_DSPMixerU16s() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  uint16_t *pun16Buff = reinterpret_cast<uint16_t *>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int16_t   an16Data[2];
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t an32Sum[2] = {0, 0};
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(an16Data)
          );
          an32Sum[0] += an16Data[0];
          an32Sum[1] += an16Data[1];
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
    register uint16_t tmp0 = 0;
    if(an32Sum[0] >= -32768) tmp0 = uint16_t(an32Sum[0]) ^ 0x8000;
    if(an32Sum[0] >   32767) tmp0 = 65535;
    *pun16Buff++ = tmp0;
    register uint16_t tmp1 = 0;
    if(an32Sum[1] >= -32768) tmp1 = uint16_t(an32Sum[1]) ^ 0x8000;
    if(an32Sum[1] >   32767) tmp1 = 65535;
    *pun16Buff++ = tmp1;
#else
         if(an32Sum[0] < -32768) *pun16Buff++ = 0;
    else if(an32Sum[0] >  32767) *pun16Buff++ = 65535;
    else                         *pun16Buff++ = uint16_t(an32Sum[0]) ^ 0x8000;
         if(an32Sum[1] < -32768) *pun16Buff++ = 0;
    else if(an32Sum[1] >  32767) *pun16Buff++ = 65535;
    else                         *pun16Buff++ = uint16_t(an32Sum[1]) ^ 0x8000;
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_DSPMixerS16m() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  int16_t *pn16Buff = reinterpret_cast<int16_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int16_t  n16Data;
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t n32Sum = 0;
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(&n16Data)
          );
          n32Sum += n16Data;
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
    register int16_t tmp = -32768;
    if(n32Sum >= -32768) tmp = int16_t(n32Sum);
    if(n32Sum >   32767) tmp = 32767;
    *pn16Buff++ = tmp;
#else
         if(n32Sum < -32768) *pn16Buff++ = -32768;
    else if(n32Sum >  32767) *pn16Buff++ =  32767;
    else                     *pn16Buff++ = int16_t(n32Sum);
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_DSPMixerS16s()
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  int16_t *pn16Buff = reinterpret_cast<int16_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int16_t  an16Data[2];
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t an32Sum[2] = {0, 0};
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(an16Data)
          );
          an32Sum[0] += an16Data[0];
          an32Sum[1] += an16Data[1];
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
    register int16_t tmp0 = -32768;
    if(an32Sum[0] >= -32768) tmp0 = int16_t(an32Sum[0]);
    if(an32Sum[0] >   32767) tmp0 = 32767;
    *pn16Buff++ = tmp0;
    register int16_t tmp1 = -32768;
    if(an32Sum[1] >= -32768) tmp1 = int16_t(an32Sum[1]);
    if(an32Sum[1] >   32767) tmp1 = 32767;
    *pn16Buff++ = tmp1;
#else
         if(an32Sum[0] < -32768) *pn16Buff++ = -32768;
    else if(an32Sum[0] >  32767) *pn16Buff++ =  32767;
    else                         *pn16Buff++ = int16_t(an32Sum[0]);
         if(an32Sum[1] < -32768) *pn16Buff++ = -32768;
    else if(an32Sum[1] >  32767) *pn16Buff++ =  32767;
    else                         *pn16Buff++ = int16_t(an32Sum[1]);
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}


//
// Endian inverted sound mixing functions
//
void CAudioEnvCImp::_IDSPMixerU16m() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  uint16_t *pun16Buff = reinterpret_cast<uint16_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int16_t   n16Data;
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t n32Sum = 0;
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(&n16Data)
          );
          n32Sum += n16Data;
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
             uint16_t un16Tmp;
    register uint16_t tmp = 0;
    if(n32Sum >= -32768) tmp = uint16_t(n32Sum) ^ 0x8000;
    if(n32Sum >   32767) tmp = 65535;
    un16Tmp = tmp; _SWAP_STORE(pun16Buff, &un16Tmp);
#else
    uint16_t un16Tmp;
         if(n32Sum < -32768) un16Tmp = 0;
    else if(n32Sum >  32767) un16Tmp = 65535;
    else                     un16Tmp = uint16_t(n32Sum) ^ 0x8000;
    _SWAP_STORE(pun16Buff, &un16Tmp);
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_IDSPMixerU16s() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  uint16_t *pun16Buff = reinterpret_cast<uint16_t *>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int16_t   an16Data[2];
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t an32Sum[2] = {0, 0};
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(an16Data)
          );
          an32Sum[0] += an16Data[0];
          an32Sum[1] += an16Data[1];
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
             uint16_t un16Tmp;
    register uint16_t tmp0 = 0;
    if(an32Sum[0] >= -32768) tmp0 = uint16_t(an32Sum[0]) ^ 0x8000;
    if(an32Sum[0] >   32767) tmp0 = 65535;
    un16Tmp = tmp0; _SWAP_STORE(pun16Buff, &un16Tmp);
    register uint16_t tmp1 = 0;
    if(an32Sum[1] >= -32768) tmp1 = uint16_t(an32Sum[1]) ^ 0x8000;
    if(an32Sum[1] >   32767) tmp1 = 65535;
    un16Tmp = tmp1; _SWAP_STORE(pun16Buff, &un16Tmp);
#else
    uint16_t un16Tmp;
         if(an32Sum[0] < -32768) un16Tmp = 0;
    else if(an32Sum[0] >  32767) un16Tmp = 65535;
    else                         un16Tmp = uint16_t(an32Sum[0]) ^ 0x8000;
    _SWAP_STORE(pun16Buff, &un16Tmp);
         if(an32Sum[1] < -32768) un16Tmp = 0;
    else if(an32Sum[1] >  32767) un16Tmp = 65535;
    else                         un16Tmp = uint16_t(an32Sum[1]) ^ 0x8000;
    _SWAP_STORE(pun16Buff, &un16Tmp);
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_IDSPMixerS16m() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  int16_t *pn16Buff = reinterpret_cast<int16_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int16_t  n16Data;
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t n32Sum = 0;
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(&n16Data)
          );
          n32Sum += n16Data;
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
             int16_t n16Tmp;
    register int16_t tmp = -32768;
    if(n32Sum >= -32768) tmp = int16_t(n32Sum);
    if(n32Sum >   32767) tmp = 32767;
    n16Tmp = tmp; _SWAP_STORE(pn16Buff, &n16Tmp);
#else
    int16_t n16Tmp;
         if(n32Sum < -32768) n16Tmp = -32768;
    else if(n32Sum >  32767) n16Tmp =  32767;
    else                     n16Tmp = int16_t(n32Sum);
    _SWAP_STORE(pn16Buff, &n16Tmp);
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

void CAudioEnvCImp::_IDSPMixerS16s() //! \bug NOT TESTED YET !!!
{
  // Check
  if(CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite == 0) return;

  // Mix sound
  int16_t *pn16Buff = reinterpret_cast<int16_t*>(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite);
  int16_t  an16Data[2];
  for(size_t smplCnt = 0; smplCnt < CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize4; ++smplCnt) {
    int32_t an32Sum[2] = {0, 0};
    for(size_t sbfCnt = 0; sbfCnt < _MAX_SB_; ++sbfCnt) {
      if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB) {
        if(CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData.bPlaying) {
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB->_GetData_(
            CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].tData,
            reinterpret_cast<u_char*>(an16Data)
          );
          an32Sum[0] += an16Data[0];
          an32Sum[1] += an16Data[1];
        }
        else
          CAudioEnvCImp::_instAudioEnv->_atPSB[sbfCnt].pcsSB = NULL;
      }
    }
#ifdef __USE_CMOV__
             int16_t n16Tmp;
    register int16_t tmp0 = -32768;
    if(an32Sum[0] >= -32768) tmp0 = int16_t(an32Sum[0]);
    if(an32Sum[0] >   32767) tmp0 = 32767;
    n16Tmp = tmp0; _SWAP_STORE(pn16Buff, &n16Tmp);
    register int16_t tmp1 = -32768;
    if(an32Sum[1] >= -32768) tmp1 = int16_t(an32Sum[1]);
    if(an32Sum[1] >   32767) tmp1 = 32767;
    n16Tmp = tmp1; _SWAP_STORE(pn16Buff, &n16Tmp);
#else
    int16_t n16Tmp;
         if(an32Sum[0] < -32768) n16Tmp = -32768;
    else if(an32Sum[0] >  32767) n16Tmp =  32767;
    else                         n16Tmp = int16_t(an32Sum[0]);
    _SWAP_STORE(pn16Buff, &n16Tmp);
         if(an32Sum[1] < -32768) n16Tmp = -32768;
    else if(an32Sum[1] >  32767) n16Tmp =  32767;
    else                         n16Tmp = int16_t(an32Sum[1]);
    _SWAP_STORE(pn16Buff, &n16Tmp);
#endif
  }

  // Decrement the "ToWrite" value
  CAudioEnvCImp::_instAudioEnv->_csMutex.Lock();
  CAudioEnvCImp::_instAudioEnv->_tCD->un8ToWrite--;
  CAudioEnvCImp::_instAudioEnv->_csMutex.Unlock();

  // Increment the write pointer
  CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite += CAudioEnvCImp::_instAudioEnv->_tCD->un32FragSize;
  if(CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite > CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffMax) CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuffWrite = CAudioEnvCImp::_instAudioEnv->_tCD->pucPBuff;
}

} // namespace vge
