/** \file AOSSEnvironment.h */
// ############################################################################################
//  AOSSEnvironment.h
//  Definition & implementation of COssAudioEnv class using OSS audio system
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#include "../../VConfig.h"
#if !defined(_AUDIO_ENVIRONMENT___PCM_OSS___AOSS_ENVIRONMENT_H_) && defined(__USE_OSS_AUDIO__)
#define _AUDIO_ENVIRONMENT___PCM_OSS___AOSS_ENVIRONMENT_H_

// VortexGE headers
#include "../NonDummyAudioEnvBaseClass.h"


namespace vge {

  // Foward reference from "lib/AudioEnvironment/PCM_OSS/AOSSSoundBuffer.h"
  class COssSoundBuffer;


  //!
  //! \brief OSS audio environment class
  //!
  //! \api pretty stable
  class VGE_EXPORT COssAudioEnv : public CAudioEnvCImp, VCLASS_SINGLETON {
    // Support for singleton class
    VCLASS_SINGLETON_DECLARE(COssAudioEnv)

    public:
      COssAudioEnv();
      virtual ~COssAudioEnv() throw();
      // Quick initialization functions
      virtual void DoQuickInit(const char *pcDeviceName = NULL);
      // Initialization functions
      virtual size_t QueryDeviceName(char *pcResult, size_t bufferSize = PATH_MAX);
      virtual void OpenDevice(const char *pcDeviceName);
      virtual void InitializeDevice(const TAudioSettings *ptPreferred = NULL, TAudioSettings *ptRealized = NULL);
      virtual void ActivateDevice();
      virtual void DeactivateDevice();
      virtual void GetAudioSpec(uint32_t *un32Channels, uint32_t *un32Bits, uint32_t *un32Rates, ESoundBufferFormat *eFormat, uint32_t *un32FragSize) const;
      virtual void SetMidiPlayer(CMidiPlayer *pcsPlayer);
      // Sound functions
      virtual bool IsAvailable() const { return(_tD.n32DSPFD >= 0); }
      virtual void SuspendDevice()     { _csTWriter.Suspend(); }
      virtual void ResumeDevice()      { _csTWriter.Resume(); }
      virtual void UpdateBuffer()      { if(_tD.n32DSPFD >= 0) _pfnDoSoundMixing(); }
      // Sound buffer functions
      virtual CSoundBuffer *NewSoundBuffer();

      // Friend classes
      friend class COssSoundBuffer;

    protected:
      // Initialization functions
      bool _AdjustDeviceName_(char *pcDeviceName);
      // Sound buffer functions
      void _ClearBuffer_(u_char *pucBuff, uint32_t un32BuffSize, uint8_t un8Sign, uint8_t un8Bits);
      // Thread functions
      static uint64_t _ThreadProc(TThreadData *ptThreadData);

      // Data
      struct _TDSPData : public _TCommonDSPData {
        char    acDevName[PATH_MAX]; // Audio device path
        int32_t n32DSPFD;            // Audio device handle
      } _tD;

      // Pointer to the one and the only COssAudioEnv instance
      static COssAudioEnv *_instance;
  };

} // namespace vge
#endif
