// ############################################################################################
//  0-CompilerCheck.cc
//  This file will perform some compile time checkings, this file will be not included
//  (archived/linked) with the resulting library
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "../2DManipulation/BitmapStructs.h"
#include "../AudioEnvironment/WaveStructs.h"
#include "../VideoEnvironment/VideoBase.h"


namespace vge {

// Used for checking the size of simple data types
#define _CHECK_SIZE_(Name, Size) typedef char _dummy##Name[int(sizeof(Name) == Size) - 1]

// Create a dummy enumeration
enum _vdenum_ {_dummy_enum_value_};

// Check if all simple data types have the correct size
_CHECK_SIZE_(char,    1); _CHECK_SIZE_(u_char,   1);
_CHECK_SIZE_(int8_t,  1); _CHECK_SIZE_(uint8_t,  1);
_CHECK_SIZE_(int16_t, 2); _CHECK_SIZE_(uint16_t, 2);
_CHECK_SIZE_(int32_t, 4); _CHECK_SIZE_(uint32_t, 4);
_CHECK_SIZE_(int64_t, 8); _CHECK_SIZE_(uint64_t, 8);
_CHECK_SIZE_(float,   4); _CHECK_SIZE_(double,   8);
_CHECK_SIZE_(bool,    1); _CHECK_SIZE_(_vdenum_, sizeof(int32_t));

// Check if some packed structures have the correct size
// (if so, the others should be also correct)
_CHECK_SIZE_(_TRGBTRIPLE,          3);
_CHECK_SIZE_(_TRGBQUAD ,           4);
_CHECK_SIZE_(_TBITMAPFILEHEADER,  14);
_CHECK_SIZE_(_TBITMAPCOREHEADER,  12);
_CHECK_SIZE_(_TBITMAPINFOHEADER,  40);
_CHECK_SIZE_(_TCWAVECHUNK,        12);
_CHECK_SIZE_(_TFORMATSUBCHUNKPCM, 24);
_CHECK_SIZE_(_TDATASUBCHUNK,       8);
_CHECK_SIZE_(_TCWFPCMHEADER,      44);
_CHECK_SIZE_(TUN8RGB,        3);

} // namespace vge

