/** \file Clipboard.h */
// ############################################################################################
//  Clipboard.h
//  Definition of CClipboard class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _VIDEO_ENVIRONMENT___CLIPBOARD_H_
#define _VIDEO_ENVIRONMENT___CLIPBOARD_H_

// VortexGE header
#include "Color.h"


namespace vge {

  // Foward reference from the video system implementation details
  // ("lib/VideoEnvironment/VID_*/*")
  class  CVideoEnv;


  //!
  //! \brief Common clipboard class applicable to all video system
  //!
  //! \api beta
  class VGE_EXPORT CClipboard {
    public:
      //!
      //! \brief Data type to be copied/pasted
      //!
      enum CDT { CDT_Null, CDT_String };

    public:
      // Initialization functions
      //!
      //! Set the video environment instance to be used as the clipboard's owner.
      inline static void SetVideoEnv(CVideoEnv *vidEnv) 
      { _vidEnv = vidEnv; }
      //!
      //! Get the video environment instance which owns the clipboard.
      inline static CVideoEnv *GetVideoEnv()
      { return(_vidEnv); }

      // Copy to clipboard functions
      //!
      //! Copy the given string to clipboard.
      static void CopyTo(const string &srcString, bool clipboard = true);

      // Copy from clipboard functions
      //!
      //! Request data of the given type from clipboard.
      static void RequestData(CDT dataType, bool clipboard = true);
      //!
      //! Extract a string data from clipboard.
      static void CopyFrom(string &dstString);

    protected:
      static CVideoEnv *_vidEnv; // The clipboard's owner

    private:
      // Make this class non instantiateable
      inline CClipboard() {}
  };

} // namespace vge
#endif
