// ############################################################################################
//  VX11Dialog3.cc
//  Implementation of dialog classes using X Window system (Xlib)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// NOTE: This file is meant to be included from within
//       "lib/VideoEnvironment/DialogEnvironment.cc"


namespace vge {

//
// List-box control
//
const uint32_t CListBox::INVALID_INDEX;

CListBox::CListBox()
: _un32CurSel  (0), _un32Total (0),
  _un32StartDsp(0), _un32MaxDsp(0), _un32DspMax(0)
{
}

CListBox::~CListBox() throw()
{
  DelAllItems();
}

void CListBox::InitControl(CVideoEnv &vidEnv, const _CControl *pcsParent,
                           int32_t n32X, int32_t n32Y, uint32_t un32Width, uint32_t un32Height)
{
  // Temporary variables
  int32_t n32Xc, n32Yc;

  if((un32Width < 28) || (un32Height < 54))
    VError(VERR_InvalidParameter, _NFL_, "List box size is too small\nThe minimum size is 28x54 pixels");

  // Basic initializations
  InitBaseData(vidEnv, pcsParent, ECID_ListBox, n32X, n32Y, un32Width, un32Height);
  n32X        = _tBD.n32X1;
  n32Y        = _tBD.n32Y1;
  _un32AH     = un32Height;
  _un32FH     = 10;
  _bUseSlider = false;
  _bRecalcFH  = true;
  _un32DspMax = (un32Height - 4) / _un32FH;
  if(_un32DspMax < 1) _un32DspMax = 1;

  // The outline
  _tOutline.x = n32X; _tOutline.width  = un32Width - 1;
  _tOutline.y = n32Y; _tOutline.height = un32Height - 1;
  n32X++; n32Y++; un32Width -= 2; un32Height -= 2;

  // The main text area
  _tMain.x = n32X + 1; _tMain.width  = un32Width - 2 + 1;
  _tMain.y = n32Y + 1; _tMain.height = un32Height - 2 + 1;
  _tSLB.x  = n32X + un32Width - 13 + 1; _tSLB.width = 11 + 1;
  _tSLB.y  = n32Y + 1;                  _tSLB.height = un32Height - 2 + 1;
  _un32Yt  = _tMain.y + (*vidEnv._tX11Style.ptFont)->ascent;
  // The selected item area
  // NB : the Y coordinate is dynamic
  _tSelItem.x = _tMain.x; _tSelItem.width  = _tMain.width - 1;
  _tSelItem.y = _tMain.y; _tSelItem.height = _un32FH + 1;

  // The main segments
  _atMain[0].x1 = n32X;                 _atMain[0].y1 = n32Y;
  _atMain[0].x2 = n32X;                 _atMain[0].y2 = n32Y + un32Height - 1;
  _atMain[1].x1 = n32X;                 _atMain[1].y1 = n32Y;
  _atMain[1].x2 = n32X + un32Width - 1; _atMain[1].y2 = n32Y;
  _atMain[2].x1 = n32X + 1;             _atMain[2].y1 = n32Y + un32Height - 1;
  _atMain[2].x2 = n32X + un32Width - 1; _atMain[2].y2 = n32Y + un32Height - 1;
  _atMain[3].x1 = n32X + un32Width - 1; _atMain[3].y1 = n32Y + 1;
  _atMain[3].x2 = n32X + un32Width - 1; _atMain[3].y2 = n32Y + un32Height - 1;

  // The up button & arrows
  _tUp.x1 = n32X + un32Width - 13; _tUp.y1 = n32Y;
  _tUp.x2 = _tUp.x1 + 13 - 1;    _tUp.y2 = _tUp.y1 + 13 - 1;
  _atUp[0].x1 = _tUp.x1;     _atUp[0].y1 = _tUp.y1;
  _atUp[0].x2 = _tUp.x1;     _atUp[0].y2 = _tUp.y2;
  _atUp[1].x1 = _tUp.x1 + 1; _atUp[1].y1 = _tUp.y1;
  _atUp[1].x2 = _tUp.x2;     _atUp[1].y2 = _tUp.y1;
  _atUp[2].x1 = _tUp.x2;     _atUp[2].y1 = _tUp.y1 + 1;
  _atUp[2].x2 = _tUp.x2;     _atUp[2].y2 = _tUp.y2;
  _atUp[3].x1 = _tUp.x1 + 1; _atUp[3].y1 = _tUp.y2;
  _atUp[3].x2 = _tUp.x2;     _atUp[3].y2 = _tUp.y2;
  n32Xc = _tUp.x1 + 6; n32Yc = _tUp.y1 + 5;
  _atArrow[0].x1 = n32Xc - 4; _atArrow[0].x2 = n32Xc;     _atArrow[0].y1 = n32Yc + 2; _atArrow[0].y2 = n32Yc - 2;
  _atArrow[1].x1 = n32Xc;     _atArrow[1].x2 = n32Xc + 4; _atArrow[1].y1 = n32Yc - 2; _atArrow[1].y2 = n32Yc + 2;
  _atArrow[4].x1 = _atArrow[0].x1; _atArrow[4].y1 = _atArrow[0].y1 + 1;
  _atArrow[4].x2 = _atArrow[0].x2; _atArrow[4].y2 = _atArrow[0].y2 + 1;
  _atArrow[5].x1 = _atArrow[1].x1; _atArrow[5].y1 = _atArrow[1].y1 + 1;
  _atArrow[5].x2 = _atArrow[1].x2; _atArrow[5].y2 = _atArrow[1].y2 + 1;

  // The down button & arrows
  _tDn.x1 = _tUp.x1;  _tDn.y1 = n32Y + un32Height - 13;
  _tDn.x2 = _tUp.x2;  _tDn.y2 = _tDn.y1 + 13 - 1;
  _atDn[0].x1 = _tDn.x1;     _atDn[0].y1 = _tDn.y1;
  _atDn[0].x2 = _tDn.x1;     _atDn[0].y2 = _tDn.y2;
  _atDn[1].x1 = _tDn.x1 + 1; _atDn[1].y1 = _tDn.y1;
  _atDn[1].x2 = _tDn.x2;     _atDn[1].y2 = _tDn.y1;
  _atDn[2].x1 = _tDn.x2;     _atDn[2].y1 = _tDn.y1 + 1;
  _atDn[2].x2 = _tDn.x2;     _atDn[2].y2 = _tDn.y2;
  _atDn[3].x1 = _tDn.x1 + 1; _atDn[3].y1 = _tDn.y2;
  _atDn[3].x2 = _tDn.x2;     _atDn[3].y2 = _tDn.y2;
  n32Xc = _tDn.x1 + 6; n32Yc = _tDn.y1 + 5;
  _atArrow[2].x1 = n32Xc - 4; _atArrow[2].x2 = n32Xc;     _atArrow[2].y1 = n32Yc - 2; _atArrow[2].y2 = n32Yc + 2;
  _atArrow[3].x1 = n32Xc;     _atArrow[3].x2 = n32Xc + 4; _atArrow[3].y1 = n32Yc + 2; _atArrow[3].y2 = n32Yc - 2;
  _atArrow[6].x1 = _atArrow[2].x1; _atArrow[6].y1 = _atArrow[2].y1 + 1;
  _atArrow[6].x2 = _atArrow[2].x2; _atArrow[6].y2 = _atArrow[2].y2 + 1;
  _atArrow[7].x1 = _atArrow[3].x1; _atArrow[7].y1 = _atArrow[3].y1 + 1;
  _atArrow[7].x2 = _atArrow[3].x2; _atArrow[7].y2 = _atArrow[3].y2 + 1;

  // The slider segments
  _tSlider.x1 = _tUp.x1;
  _tSlider.y1 = _tUp.y2 + 1;
  _tSlider.x2 = _tDn.x2;
  _tSlider.y2 = _tDn.y1 - 1;
  _atSlider[0].x1 = _tSlider.x1;     _atSlider[0].y1 = _tSlider.y1;
  _atSlider[0].x2 = _tSlider.x1;     _atSlider[0].y2 = _tSlider.y2;
  _atSlider[1].x1 = _tSlider.x1 + 1; _atSlider[1].y1 = _tSlider.y1;
  _atSlider[1].x2 = _tSlider.x2;     _atSlider[1].y2 = _tSlider.y1;
  _atSlider[6].x1 = _tSlider.x1 + 1; _atSlider[6].y1 = _tSlider.y2;
  _atSlider[6].x2 = _tSlider.x2;     _atSlider[6].y2 = _tSlider.y2;
  _atSlider[7].x1 = _tSlider.x2;     _atSlider[7].y1 = _tSlider.y1 + 1;
  _atSlider[7].x2 = _tSlider.x2;     _atSlider[7].y2 = _tSlider.y2;

  // The slider tracks
  _atSlider[2].x1 = _tSlider.x1 + 5;     _atSlider[2].y1 = _tSlider.y1 + 3;
  _atSlider[2].x2 = _atSlider[2].x1;     _atSlider[2].y2 = _tSlider.y2 - 3;
  _atSlider[3].x1 = _atSlider[2].x1 + 3; _atSlider[3].y1 = _atSlider[2].y1;
  _atSlider[3].x2 = _atSlider[3].x1;     _atSlider[3].y2 = _atSlider[2].y2;
  _atSlider[8].x1 = _atSlider[2].x1 - 1; _atSlider[8].y1 = _atSlider[2].y1;
  _atSlider[8].x2 = _atSlider[8].x1;     _atSlider[8].y2 = _atSlider[2].y2;
  _atSlider[9].x1 = _atSlider[8].x1 + 3; _atSlider[9].y1 = _atSlider[8].y1;
  _atSlider[9].x2 = _atSlider[9].x1;     _atSlider[9].y2 = _atSlider[8].y2;

  // The slider levers
  // NB : the Y coordinates are dynamic
  _atSlider[4].x1  = _tSlider.x1 + 3;     _atSlider[4].x2  = _atSlider[4].x1;
  _atSlider[5].x1  = _atSlider[4].x1 + 1; _atSlider[5].x2  = _atSlider[5].x1 + 5;
  _atSlider[10].x1 = _atSlider[4].x1 + 1; _atSlider[10].x2 = _atSlider[10].x1 + 5;
  _atSlider[11].x1 = _atSlider[5].x2;     _atSlider[11].x2 = _atSlider[11].x1;
  _atSlider[12].x1 = _atSlider[5].x1;     _atSlider[12].x2 = _atSlider[12].x1 + 4;
  _atSlider[4].y1  = _tSlider.y1 + 2;     _atSlider[4].y2  = _atSlider[4].y1 + 2;
  _atSlider[5].y1  = _atSlider[4].y1;     _atSlider[5].y2  = _atSlider[5].y1;
  _atSlider[10].y1 = _atSlider[4].y2;     _atSlider[10].y2 = _atSlider[10].y1;
  _atSlider[11].y1 = _atSlider[4].y1 + 1; _atSlider[11].y2 = _atSlider[4].y2;
  _atSlider[12].y1 = _atSlider[11].y1;    _atSlider[12].y2 = _atSlider[12].y1;
}

void CListBox::AddItem(const string &newItem)
{
  _sItems.push_back(newItem);
  _un32Total++;
  _un32MaxDsp = _un32Total - _un32DspMax;
#ifdef __USE_UTF8_DIALOG_STRING__
  _bRecalcFH = true;
#endif
}

void CListBox::AddItems(const vector<string> &newItems)
{
  for(size_t i = 0; i < newItems.size(); ++i) AddItem(newItems[i]);
}

void CListBox::AddItems(uint32_t un32ItemCount, const char *pcItem, ...)
{
  // Add items
  if(pcItem) {
    va_list vlTmp;
    AddItem(pcItem);
    va_start(vlTmp, pcItem);
    for(size_t i = 0; i < (un32ItemCount - 1); ++i) {
      const char *pcTmp = va_arg(vlTmp, char *);
      if(pcTmp) AddItem(pcTmp);
    }
    va_end(vlTmp);
  }
}

void CListBox::DelItem(uint32_t un32ItemIndex)
{
  // Check
  if(_sItems.empty()) return;
  if(un32ItemIndex >= _un32Total)
    VError(VERR_InvalidParameter, _NFL_, "Item index to be deleted is out of range");
  if(_un32Total == 1) {
    DelAllItems();
    return;
  }

  // Delete item
  _sItems.erase(_sItems.begin() + un32ItemIndex);

  // Update variables
  _un32Total--;
  if(_un32CurSel >= _un32Total) _un32CurSel = _un32Total - 1;
  if(_un32Total > _un32DspMax) {
    _un32MaxDsp = _un32Total - _un32DspMax;
    if(_un32StartDsp > _un32MaxDsp) _un32StartDsp = _un32MaxDsp;
  }
  else {
    _un32MaxDsp = 0;
    _un32StartDsp = 0;
  }
#ifdef __USE_UTF8_DIALOG_STRING__
  _bRecalcFH = true;
#endif
}

void CListBox::DelItem(const string &item, bool bCaseSensitive)
{
  if(_un32Total == 0) return;

  uint32_t tmp;
  if(FindItem(&tmp, item, bCaseSensitive)) DelItem(tmp);
}

void CListBox::DelItems(uint32_t un32IndexCount, uint32_t un32ItemIndex, ...)
{
  // Delete items
  va_list vlTmp;
  DelItem(un32ItemIndex);
  va_start(vlTmp, un32ItemIndex);
    for(size_t i = 0; i < un32IndexCount; ++i)
      DelItem(va_arg(vlTmp, uint32_t));
  va_end(vlTmp);
}

void CListBox::DelItems(uint32_t un32IndexCount, const uint32_t *pun32ItemIndexes)
{
  // Delete items
  for(size_t i = 0; i < un32IndexCount; i++)
    DelItem(pun32ItemIndexes[i]);
}

void CListBox::DelItems(const vector<string> &items, bool bCaseSensitive)
{
  if(_un32Total == 0) return;

  uint32_t tmp;
  for(size_t i = 0; i < items.size(); ++i) 
    if(FindItem(&tmp, items[i], bCaseSensitive)) DelItem(tmp);
}

void CListBox::DelAllItems()
{
  // Delete all
  _sItems.clear();

  // Set variables
  _un32CurSel   = 0;
  _un32Total    = 0;
  _un32StartDsp = 0;
  _un32MaxDsp   = 0;
#ifdef __USE_UTF8_DIALOG_STRING__
  _bRecalcFH    = true;
#endif
}

uint32_t CListBox::GetSelectedIndex() const
{
  if(!_un32Total) return(0);
  return(_un32CurSel);
}

const char *CListBox::GetSelectedItem() const
{
  if(!_un32Total) return(0);
  return(_sItems[_un32CurSel].c_str());
}

void CListBox::SelectItem(uint32_t un32ItemIndex)
{
  if(_un32Total == 0) return;
  _un32CurSel = un32ItemIndex;
  if(_un32CurSel >= _un32Total) _un32CurSel = _un32Total - 1;
}

void CListBox::SelectItem(const string &item, bool bCaseSensitive)
{
  if(_un32Total == 0) return;

  uint32_t tmp;
  if(FindItem(&tmp, item, bCaseSensitive)) _un32CurSel = tmp;
}

bool CListBox::FindItem(uint32_t *pun32RetIndex, const string &item, bool bCaseSensitive) const
{
  // Find item
  if(bCaseSensitive) {
    for(size_t i = 0; i < _un32Total; ++i)
    if(_sItems[i] == item) {
      if(pun32RetIndex) *pun32RetIndex = i;
      return(true);
    }
  }
  else {
    for(size_t i = 0; i < _un32Total; ++i)
    if(strcasecmp(_sItems[i].c_str(), item.c_str()) == 0) {
      if(pun32RetIndex) *pun32RetIndex = i;
      return(true);
    }
  }
  if(pun32RetIndex) *pun32RetIndex = 0;
  return(false);
}

bool CListBox::ToggleValue(int32_t n32MouseX, int32_t n32MouseY)
{
  // Temporary variables
  uint32_t un32T;

  // Check
  if(!_tBD.bEnabled) return(false);

  // Check up buttons
  if(_bUseSlider && MouseOver(_tUp.x1, _tUp.y1, _tUp.x2, _tUp.y2, n32MouseX, n32MouseY)) {
    if(_un32StartDsp > 0) _un32StartDsp--;
    return(true);
  }
  // Check down buttons
  else if(_bUseSlider && MouseOver(_tDn.x1, _tDn.y1, _tDn.x2, _tDn.y2, n32MouseX, n32MouseY)) {
    if(_un32StartDsp < _un32MaxDsp) _un32StartDsp++;
    return(true);
  }
  // Check slider area
  else if(_bUseSlider && MouseOver(_tSlider.x1, _tSlider.y1, _tSlider.x2, _tSlider.y2, n32MouseX, n32MouseY)) {
    if(n32MouseY < _atSlider[4].y1) {
      un32T = (_atSlider[4].y1 - n32MouseY) * _un32MaxDsp / (_tSlider.y2 - _tSlider.y1);
      if(_un32StartDsp > un32T)
        _un32StartDsp -= un32T;
      else
        _un32StartDsp = 0;
      return(true);
    }
    else if(n32MouseY > _atSlider[4].y1) {
      _un32StartDsp += ((n32MouseY - _atSlider[4].y1) * _un32MaxDsp / (_tSlider.y2 - _tSlider.y1));
      if(_un32StartDsp > _un32MaxDsp) _un32StartDsp = _un32MaxDsp;
      return(true);
    }
  }
  // Check text area
  else if(MouseOver(_tMain.x, _tMain.y, _tMain.x + _tMain.width, _tMain.y + _tMain.height, n32MouseX, n32MouseY)) {
    _un32CurSel = _un32StartDsp + (n32MouseY - _tMain.y) / _un32FH;
    if(_un32CurSel >= _un32Total) _un32CurSel = _un32Total - 1;
    return(true);
  }
  return(false);
}

void CListBox::DrawControl() const
{
  DrawControl(false, false, false, 0);
}

void CListBox::DrawControl(bool bUpPressed, bool bDnPressed, bool bSlPressed, uint32_t un32TxPressed) const
{
  CVideoEnv &vidEnv = *_tBD.pcsVidEnv;

  // Recalculate some of the properties if needed
  if(_bRecalcFH) {
    _bRecalcFH = false;
#ifndef __USE_UTF8_DIALOG_STRING__
    _un32FH = (*vidEnv._tX11Style.ptFont)->ascent + (*vidEnv._tX11Style.ptFont)->descent;
#else
    for(size_t i = 0; i < _un32Total; i++) {
      XRectangle tl;
      Xutf8TextExtents(*vidEnv._tX11Style.ptFontSet, _sItems[i].data(), _sItems[i].length(), NULL, &tl);
      _un32FH = max<uint32_t>(_un32FH, tl.height);
    }
#endif
    _un32DspMax = (_un32AH - 4) / _un32FH;
    _bUseSlider = _un32Total > _un32DspMax;
  }

  // Draw outline
  vidEnv.SetForegroundColor(vidEnv._tX11Style.xOutline);
  XDrawRectangle(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                 _tOutline.x, _tOutline.y, _tOutline.width, _tOutline.height);

  // Draw main text area
  vidEnv.SetFillStyle(FILL_SOLID);
  vidEnv.SetForegroundColor(vidEnv._tX11Style.xFace);
  XFillRectangle(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                 _tMain.x, _tMain.y, _tMain.width, _tMain.height);
  vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
  XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                const_cast<XSegment*>(&_atMain[2]), 2);
  vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
  XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                const_cast<XSegment*>(_atMain), 2);

  // Draw selected item area
  if((_un32Total > 0) && (_un32CurSel >= _un32StartDsp) && (_un32CurSel < (_un32StartDsp + _un32DspMax))) {
    // Update coordinate
    _tSelItem.y = _tMain.y + _un32FH * (_un32CurSel - _un32StartDsp);
    // Draw
    if(_tBD.bEnabled) vidEnv.SetForegroundColor(vidEnv._tX11Style.xFaceSelect);
    XFillRectangle(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                    _tSelItem.x, _tSelItem.y, _tSelItem.width, _tSelItem.height);
  }

  // Draw texts
  if(_un32Total > 0) {
    uint32_t un32P = _un32Yt;
#ifndef __USE_UTF8_DIALOG_STRING__
    vidEnv.SetFont(vidEnv._tX11Style.ptFont);
#endif
    if(_tBD.bEnabled) {
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xText);
      if(un32TxPressed > 0) {
        un32TxPressed--;
        for(uint32_t un32Cnt = 0; un32Cnt < _un32DspMax; un32Cnt++) {
          uint32_t un32I = _un32StartDsp + un32Cnt;
          if(un32I >= _un32Total) break;
          if(un32Cnt == un32TxPressed) {
            vidEnv.SetForegroundColor(vidEnv._tX11Style.xHotPress);
#ifndef __USE_UTF8_DIALOG_STRING__
            XDrawString(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                        _tMain.x, un32P, _sItems[un32I].data(), _sItems[un32I].length());
#else
            Xutf8DrawString(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap,
                            *vidEnv._tX11Style.ptFontSet, vidEnv._tX11.tMainGC,
                            _tMain.x, un32P, _sItems[un32I].data(), _sItems[un32I].length());
#endif
            vidEnv.SetForegroundColor(vidEnv._tX11Style.xText);
          }
          else
#ifndef __USE_UTF8_DIALOG_STRING__
            XDrawString(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                        _tMain.x, un32P, _sItems[un32I].data(), _sItems[un32I].length());
#else
            Xutf8DrawString(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap,
                            *vidEnv._tX11Style.ptFontSet, vidEnv._tX11.tMainGC,
                            _tMain.x, un32P, _sItems[un32I].data(), _sItems[un32I].length());
#endif
          un32P += _un32FH;
        }
      }
      else {
        for(uint32_t un32Cnt = 0; un32Cnt < _un32DspMax; un32Cnt++) {
          uint32_t un32I = _un32StartDsp + un32Cnt;
          if(un32I >= _un32Total) break;
#ifndef __USE_UTF8_DIALOG_STRING__
          XDrawString(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                      _tMain.x, un32P, _sItems[un32I].data(), _sItems[un32I].length());
#else
          Xutf8DrawString(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap,
                          *vidEnv._tX11Style.ptFontSet, vidEnv._tX11.tMainGC,
                          _tMain.x, un32P, _sItems[un32I].data(), _sItems[un32I].length());
#endif
          un32P += _un32FH;
        }
      }
    }
    else {
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
      for(uint32_t un32Cnt = 0; un32Cnt < _un32DspMax; un32Cnt++) {
        uint32_t un32I = _un32StartDsp + un32Cnt;
        if(un32I >= _un32Total) break;
#ifndef __USE_UTF8_DIALOG_STRING__
        XDrawString(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                    _tMain.x, un32P, _sItems[un32I].data(), _sItems[un32I].length());
#else
        Xutf8DrawString(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap,
                        *vidEnv._tX11Style.ptFontSet, vidEnv._tX11.tMainGC,
                        _tMain.x, un32P, _sItems[un32I].data(), _sItems[un32I].length());
#endif
        un32P += _un32FH;
      }
    }
  }

  // Draw buttons and slider
  if(_bUseSlider) {
    // Fill the area
    vidEnv.SetForegroundColor(vidEnv._tX11Style.xFace);
    if(vidEnv._tX11Style.xFillStyle != FILL_GRADIENT) {
      XFillRectangle(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                     _tSLB.x, _tSLB.y, _tSLB.width, _tSLB.height);
    }
    else {
      vidEnv.DrawGradientBox(_tSLB.x, _tSLB.y, _tSLB.width, _tSLB.height,
                                 vidEnv._tX11Style.xLightBorder,
                                 vidEnv._tX11Style.xDarkBorder,
                                 vidEnv._tX11Style.xFace);
    }

    // Draw buttons
    if(_tBD.bEnabled) {
      // Draw arrows
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xText);
      XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                    const_cast<XSegment*>(_atArrow), 8);
      // Draw up button
      if(bUpPressed) {
        vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
        XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                      const_cast<XSegment*>(_atUp), 2);
        vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
        XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                      const_cast<XSegment*>(&_atUp[2]), 2);
      }
      else {
        vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
        XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                      const_cast<XSegment*>(_atUp), 2);
        vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
        XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                      const_cast<XSegment*>(&_atUp[2]), 2);
      }
      // Draw down button
      if(bDnPressed) {
        vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
        XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                      const_cast<XSegment*>(_atDn), 2);
        vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
        XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                      const_cast<XSegment*>(&_atDn[2]), 2);
      }
      else {
        vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
        XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                      const_cast<XSegment*>(_atDn), 2);
        vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
        XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                      const_cast<XSegment*>(&_atDn[2]), 2);
      }
    }
    else {
      // Draw arrows
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
      XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                    const_cast<XSegment*>(&_atArrow[4]), 4);
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
      XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                    const_cast<XSegment*>(_atArrow), 4);
      // Draw up button
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
      XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                   const_cast<XSegment*>(_atUp), 2);
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
      XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                    const_cast<XSegment*>(&_atUp[2]), 2);
      // Draw down button
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
      XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                    const_cast<XSegment*>(_atDn), 2);
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
      XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                    const_cast<XSegment*>(&_atDn[2]), 2);
    }

    // Update the slider coordinates
    _atSlider[4].y1  = _tSlider.y1 + (_tSlider.y2 - _tSlider.y1 - 6) * _un32StartDsp / _un32MaxDsp + 2;
    if(_atSlider[4].y1 > (_tSlider.y2 - 2)) _atSlider[4].y1 = _tSlider.y2 - 2;
    _atSlider[4].y2  = _atSlider[4].y1 + 2;
    _atSlider[5].y1  = _atSlider[4].y1;     _atSlider[5].y2  = _atSlider[5].y1;
    _atSlider[10].y1 = _atSlider[4].y2;     _atSlider[10].y2 = _atSlider[10].y1;
    _atSlider[11].y1 = _atSlider[4].y1 + 1; _atSlider[11].y2 = _atSlider[4].y2;
    _atSlider[12].y1 = _atSlider[11].y1;    _atSlider[12].y2 = _atSlider[12].y1;

    // Draw slider
    vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
    XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                  const_cast<XSegment*>(_atSlider), 4);
    vidEnv.SetForegroundColor(vidEnv._tX11Style.xDarkBorder);
    XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                  const_cast<XSegment*>(&_atSlider[6]), 6);
    vidEnv.SetForegroundColor(vidEnv._tX11Style.xLightBorder);
    XDrawSegments(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
                  const_cast<XSegment*>(&_atSlider[4]), 2);
    if(_tBD.bEnabled && bSlPressed)
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xHotPress);
    else
      vidEnv.SetForegroundColor(vidEnv._tX11Style.xFace);
    XDrawLine(vidEnv._tX11.ptDisplay, vidEnv._tX11.tMainPixmap, vidEnv._tX11.tMainGC,
      _atSlider[12].x1, _atSlider[12].y1, _atSlider[12].x2, _atSlider[12].y2);
  }
}

void CListBox::DrawControl(bool bUpPressed, bool bDnPressed) const
{
  DrawControl(bUpPressed, bDnPressed, false, 0);
}

void CListBox::DrawControl(int32_t n32MouseX, int32_t n32MouseY, bool bPressed) const
{
  if(_bUseSlider && MouseOver(_tUp.x1, _tUp.y1, _tUp.x2, _tUp.y2, n32MouseX, n32MouseY))
    DrawControl(bPressed, false, false, 0);
  else if(_bUseSlider && MouseOver(_tDn.x1, _tDn.y1, _tDn.x2, _tDn.y2, n32MouseX, n32MouseY))
    DrawControl(false, bPressed, false, 0);
  else if(_bUseSlider && MouseOver(_tSlider.x1, _tSlider.y1, _tSlider.x2, _tSlider.y2, n32MouseX, n32MouseY))
    DrawControl(false, false, bPressed, 0);
  else if(MouseOver(_tMain.x, _tMain.y, _tMain.x + _tMain.width, _tMain.y + _tMain.height, n32MouseX, n32MouseY)) {
    if(bPressed)
      DrawControl(false, false, false, (n32MouseY - _tMain.y) / _un32FH + 1);
    else
      DrawControl(false, false, false, 0);
  }
  else
    DrawControl(false, false, false, 0);
}


//
// Drop-down -list control
//
//! \todo WRITE ME !!!


//
// Pop-up-menu control
//
//! \todo WRITE ME !!!


//
// Menu-bar control
//
//! \todo WRITE ME !!!

} // namespace vge
