/**
 * \file VX11Utilities.h
 * \internal
 */
// ############################################################################################
//  VX11Utilities.h
//  Prototype of various X11 utilities
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// NOTE: This file is meant to be included from within the
//       X11 video system implementation


#ifndef _VIDEO_ENVIRONMENT___VID_X11___X11_UTILITIES_H_
#define _VIDEO_ENVIRONMENT___VID_X11___X11_UTILITIES_H_

// VortexGE headers
#include "../../System/Thread.h"

// X11 headers
#ifdef __USE_XF86VM__
#  include <X11/extensions/xf86vmode.h>
#endif


namespace vge {

#ifdef __USE_XF86VM__
  typedef XF86VidModeModeInfo __intexp_XF86ModeInfo_;
#else
  typedef uint32_t __intexp_XF86ModeInfo_;
#endif

  // For saving video mode
#ifdef __USE_XF86VM__
  struct __intexp_XF86SavedMode_ {
    XF86VidModeModeLine tMode;
    int32_t             n32DotClock;
    int32_t             n32ViewX;
    int32_t             n32ViewY;
  };
#else
  struct __intexp_XF86SavedMode_ {
    uint8_t __un8Dummy__;
  };
#endif


  // XFree86 utilities
  extern void __intexp_XF86QueryVideoModes_(Display *ptDisplay, int32_t n32Screen, __intexp_XF86ModeInfo_ ***ppptModeList, int32_t *pn32ModeCount);
  extern int32_t __intexp_XF86FindNearestVideoMode_(uint32_t un32XRes, uint32_t un32YRes, __intexp_XF86ModeInfo_ **pptModeList, int32_t n32ModeCount);
  extern void __intexp_XF86SwitchToVideoMode_(Display *ptDisplay, int32_t n32Screen, __intexp_XF86ModeInfo_ *ptModeInfo);
  extern void __intexp_XF86DumpVideoModes_(__intexp_XF86ModeInfo_ **pptModeList, int32_t n32ModeCount);
  extern void __intexp_XF86FreeVideoModes_(__intexp_XF86ModeInfo_ **pptModeList, int32_t n32ModeCount);
  extern void __intexp_XF86SaveMode_(Display *ptDisplay, int32_t n32Screen, __intexp_XF86SavedMode_ *ptSavedMode);
  extern void __intexp_XF86RestoreMode_(Display *ptDisplay, int32_t n32Screen, __intexp_XF86SavedMode_ *ptSavedMode);

  // Often used X11 internal atoms
  namespace xprv {
    extern Atom _x11Atom_WM_NAME;
    extern Atom _x11Atom_WM_ICON_NAME;
    extern Atom _x11Atom_WM_TRANSIENT_FOR;
    extern Atom _x11Atom_WM_PROTOCOLS;
    extern Atom _x11Atom_WM_DELETE_WINDOW;
    extern Atom _x11Atom_NET_WM_TYPE;
    extern Atom _x11Atom_NET_WM_NAME;
    extern Atom _x11Atom_NET_WM_ICON_NAME;
    extern Atom _x11Atom_NET_WM_STATE;
    extern Atom _x11Atom_NET_WM_PID;
    extern Atom _x11Atom_CLIPBOARD;
    extern Atom _x11Atom_TARGETS;
  }
  extern void __intexp_X11InitInternalAtoms(Display *display);

  // X11 utilities used for copy 'n paste
  namespace xprv {
    extern CMutex          _selectionMutex;
    extern Window          _x11SelectionOwner;
    extern Window          _x11SelectionRequestor;
    extern CClipboard::CDT _x11SelectionDataToGet;
    extern CClipboard::CDT _x11SelectionDataType;
    extern string          _x11SelectionData_String;
  }
  extern void __intexp_X11CopyStringToClipboard(Display *display, Window owner, const string &srcText, bool clipboard = true);
  extern void __intexp_X11CopyStringFromClipboard_Request(Display *display, Window requestor, bool clipboard = true);
  extern void __intexp_X11CopyStringFromClipboard_Acquire(string &dstText);

  // Other X11 utilities
  extern bool __intexp_X11GetDesktopGeometry(Display *ptDisplay, int32_t n32Screen, uint32_t &x, uint32_t &y, uint32_t &w, uint32_t &h);
  extern bool __intexp_X11GetWindowFrameSize(Display *ptDisplay, Window tWindow, uint32_t &l, uint32_t &r, uint32_t &t, uint32_t &b);
  extern Window __intexp_X11SelectWindow_(Display *ptDisplay, Window tRootWindow, uint32_t un32XCShape = XC_diamond_cross);
  extern pid_t __intexp_X11GetOwnerPID_(Display *ptDisplay, Window tWindow);
  extern uint16_t __intexp_XKeySym2UCS_(uint16_t xkeysym);

} // namespace vge
#endif
