// ############################################################################################
//  roomedit-DlgPages-Main-CtlEH.cc
//  Implementation of roomedit's CPage_Main class (control's event handlers)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// roomedit's headers
#include "roomedit-DlgPages-Main.h"
#include "roomedit-RoomEdit.h"

// C++ headers
#include <sstream>


//
// Our main dialog page: controls' event handlers
//
void CPage_Main::_onMenuFileNew(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _app.pageGSZ.SetFuncToInvoke(&CPage_Main::_onMenuFileNew);
  _app.SelectPage(CRoomEdit::PAGE_GSZ);
}

void CPage_Main::_onMenuFileNew(const TFloatXYZ &gridSize)
{
  _onMenuFileNew("", gridSize);
}

void CPage_Main::_onMenuFileNew(const string &fileName, const TFloatXYZ &gridSize)
{
  _iVideoEnv->UseDefaultWaitCursor();

  // Clean up
  _app.SetWindowTitle("");
  _roomBuilder.CloseRoom();
  _roomFileName.clear();
  lstObjects.DelAllItems();

  // Reinitialize
  _app.SetWindowTitle(fileName.empty() ? "untitled" : fileName);
  _roomBuilder.NewRoom(gridSize);
  _roomFileName = fileName;
  _Generate3DGrid();

  _loc.curPos = TFloatXYZ(0,  0,     0);
  _SelectDefCamPos(0);

  ResetAllControls();
  UpdateAllControlsEnableStates();
  RedrawAllControls();
  MarkAsDirty();

  _iVideoEnv->UseDefaultArrowCursor();
}

void CPage_Main::_onMenuFileAbout(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  stringstream ss;
  ss << _app.WINDOW_TITLE << " " << CRoomEdit::TEXT_VERSION     << endl;
  ss << _M("Copyright (C)") << " 2004-2006 Aloysius Indrayanto" << endl << endl << endl;
  ss << _M("This program is part of VortexGE.")                 << endl;
  ss << _M("This program is licensed under the GNU LGPL.");

  DisplayTooltip("");
  _vStrg.StoreCurrentView();
  _iVideoEnv->MessageBox(_M("About \"VortexGE Room Editor\""), ss.str());
  _vStrg.RestoreCurrentView();
}

void CPage_Main::_onLstObjects(_CControl *ctrl, bool doubleClick)
{
  UNUSED_PARAMS_2(ctrl, doubleClick);

  // Make sure that the cursore moved to the newly selected object
  if(rdoMovCurObj.GetValue())
    _onRdoMovCurObj(NULL);
  // Just mark as dirty so that the scene will be re-rendererd
  else
    MarkAsDirty();
}

void CPage_Main::_onBtnEditObject(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  if(_activeObjectTab == OBJECT_TAB_PLANE) _onBtnEditObject_Plane();
}

void CPage_Main::_onBtnRenObject(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  if(_activeObjectTab == OBJECT_TAB_PLANE) _onBtnRenObject_Plane();
}

void CPage_Main::_onBtnAlignObject(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  DisplayTooltip("");
  _vStrg.StoreCurrentView();
  VNotImplementedMsgBox(*_iVideoEnv, _M("VortexGE Room Editor"));
  _vStrg.RestoreCurrentView();
}

void CPage_Main::_onBtnCloneObject(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  DisplayTooltip("");
  _vStrg.StoreCurrentView();
  VNotImplementedMsgBox(*_iVideoEnv, _M("VortexGE Room Editor"));
  _vStrg.RestoreCurrentView();
}

void CPage_Main::_onBtnDelObject(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  if(_activeObjectTab == OBJECT_TAB_PLANE) _onBtnDelObject_Plane();
}

void CPage_Main::_onRdoCamFollowFree(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  MarkAsDirty();
}

void CPage_Main::_onChkCamFreeTrg(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  if(chkCamFreeTrg.GetValue()) 
    _camTarget.SetColor(0, 31, 0);
  else
    _camTarget.SetColor(31, 31, 31);
  MarkAsDirty();
}

void CPage_Main::_onRdoMovCurCam(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  if(rdoMovCurOnly.GetValue()) {
    _cursor.SetColor(31, 31, 0);
    MarkAsDirty();
    return;
  }
  else if(rdoMovCam.GetValue()) {
    _cursor.SetColor(31, 31, 31);
    MarkAsDirty();
    return;
  }
}

void CPage_Main::_onRdoMovCurObj(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _cursor.SetColor(31, 0, 0);
  if(_activeObjectTab == OBJECT_TAB_PLANE) _onRdoMovCurObj_Plane();
  MarkAsDirty();
}

void CPage_Main::_onBtnGrid(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _app.pageGSZ.SetGridSize(_roomBuilder.GetRoomGridSize());
  _app.pageGSZ.SetFuncToInvoke(&CPage_Main::_onBtnGrid);
  _app.SelectPage(CRoomEdit::PAGE_GSZ);
}

void CPage_Main::_onBtnGrid(const TFloatXYZ &gridSize)
{
  _roomBuilder.SetRoomGridSize(gridSize);
  _Generate3DGrid();

  RedrawAllControls();
  MarkAsDirty();
}

void CPage_Main::_onBtnHelp(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  _app.SelectPage(CRoomEdit::PAGE_HELP);
}

void CPage_Main::_onBtnResetCamPos(_CControl *ctrl)
{
  for(size_t i = 0; i < elemsize(btnResetCamPos); ++ i) {
    if(&btnResetCamPos[i] == ctrl) {
      _SelectDefCamPos(i);
      MarkAsDirty();
    }
  }
}
