// ############################################################################################
//  roomedit-DlgPages-PNS.cc
//  Implementation of roomedit's CPage_PNS class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// roomedit's headers
#include "roomedit-DlgPages-PNS.h"
#include "roomedit-OPic.h"
#include "roomedit-RoomEdit.h"


// Used for converting between selected radio button and
// EPlaneDir enumeration's values
static const EPlaneDir _ePlaneDirs[6] = { EPD_0P0, EPD_0N0,
                                          EPD_P00, EPD_N00,
                                          EPD_00N, EPD_00P};


//
// Our dialog page: plane settings
//
CPage_PNS::CPage_PNS(CPage_Main &mainPage)
: CDialogPage(mainPage.GetAppInst(), &mainPage.GetAppInst()),
  _app       (mainPage.GetAppInst()),
  _mPage     (mainPage)
{
  _vStrg.Initialize(_iVideoEnv, 1);
}

CPage_PNS::~CPage_PNS() throw()
{
}

void CPage_PNS::SetPlaneSettings(const TInit_PlaneSettings *init)
{
  if(!init) {
    txtName.SetEnable(true);
    return;
  }

  txtName.SetEnable(false);
  txtName.SetText(init->planeName.c_str());
  spnRow.SetValue(init->planeRowSize);
  spnCol.SetValue(init->planeColSize);

  for(size_t i = 0; i < elemsize(rdoDirs); ++i) rdoDirs[i].SetValue(false);
  for(size_t i = 0; i < elemsize(rdoDirs); ++i) {
    if(init->planeDir == _ePlaneDirs[i]) {
      rdoDirs[i].SetValue(true);
      break;
    }
  }
  _onRdoDirs(NULL);
}

void CPage_PNS::_onBtnOK(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  const char    *n = txtName.GetText();
  const uint32_t l = txtName.GetTextLen();

  // No need to check the name if we are editing current plane's settings
  if(txtName.GetEnable()) {
    if(!l) {
      _vStrg.StoreCurrentView();
      _iVideoEnv->MessageBox(_M("Invalid Parameter"), _M("Please enter the plane name"),
                            EMBI_Error, EMBB_OK, EMBS_Tiny);
      _vStrg.RestoreCurrentView();
      return;
    }
    if(_mPage.PlaneIsExist(n)) {
      _vStrg.StoreCurrentView();
      _iVideoEnv->MessageBox(_M("Invalid Parameter"), _M("A plane with the same name is already exist"),
                            EMBI_Error, EMBB_OK, EMBS_Tiny);
      _vStrg.RestoreCurrentView();
      return;
    }
  }

  // Determine the plane's direction
  EPlaneDir d = EPD_0P0;
  for(size_t i = 0; i < elemsize(rdoDirs); ++i) {
    if(rdoDirs[i].GetValue()) {
      d = _ePlaneDirs[i];
      break;
    }
  }

  // Set the initialization data
  TInit_PlaneSettings init;
  init.planeName    = n;
  init.planeRowSize = spnRow.GetValue();
  init.planeColSize = spnCol.GetValue();
  init.planeDir     = d;

  (_mPage.*_pfnTI)(init);
  _app.SelectPage(CRoomEdit::PAGE_MAIN);
}

void CPage_PNS::_onBtnCancel(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _app.SelectPage(CRoomEdit::PAGE_MAIN);
}

void CPage_PNS::_onRdoDirs(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  static const char *rdoDir_text[] = { ">> As if it is the top side of a 3D cube",
                                       ">> As if it is the bottom side of a 3D cube",
                                       ">> As if it is the right side of a 3D cube",
                                       ">> As if it is the left side of a 3D cube",
                                       ">> As if it is the front side of a 3D cube",
                                       ">> As if it is the back side of a 3D cube" };

  for(size_t i = 0; i < elemsize(rdoDirs); ++i) {
    if(rdoDirs[i].GetValue()) {
      lblDirInfo.SetString(_M(rdoDir_text[i]));
      RedrawAllControls();
      return;
    }
  }
}

void CPage_PNS::InitializeAllControls()
{
  const uint32_t ww = _iVideoEnv->WindowWidth();
  const uint32_t wh = _iVideoEnv->WindowHeight();
  const uint32_t pw = 377;
  const uint32_t ph = 400;
  const uint32_t px = (ww - pw) / 2;
  const uint32_t py = (wh - ph) / 2;

  // Main section
  InitControl(pbrMain, NULL, px, py, pw, ph, _M("Plane Settings"));
  InitControl(hlyMain, &pbrMain);
      hlyMain.AddBlocksEx(3, 55, 0, 25);
  InitControl(vlyBtn, hlyMain, 2, 0);
      vlyBtn.AddBlocks(2);
  InitControl(btnOK, vlyBtn, 0, _M("OK"), false, true);
      Connect(btnOK, CDP_SLOT(CPage_PNS, _onBtnOK));
  InitControl(btnCancel, vlyBtn, 1, _M("Cancel"), true);
      Connect(btnCancel, CDP_SLOT(CPage_PNS, _onBtnCancel));

  // Name section
  InitControl(hlyName, hlyMain, 0, 0, 0);
      hlyName.AddBlocksEx(3, 20, 0, 10);
  InitControl(lblName, hlyName, 0, _M("Name of the plane:"));
  InitControl(txtName, hlyName, 1);

  // Settings section
  InitControl(fraSettings, hlyMain, 1, _M("Plane settings"));
  InitCompound(hvgSettings, hlyMain, 1, 10, 0);
      hvgSettings.AddRowsEx(6, 30, 8, 15, 0, 5, 15);
      hvgSettings[0].AddBlocksEx(5, 0, 60, 20, 0, 60);
  InitControl(lblRow, hvgSettings[0], 0, _M("Row size:"), EHTA_Right, EVTA_Center);
  InitControl(spnRow, hvgSettings[0], 1);
      spnRow.SetValueProp(1, 50, 1);
      spnRow.SetValue(1);
  InitControl(lblCol, hvgSettings[0], 3, _M("Column size:"), EHTA_Right, EVTA_Center);
  InitControl(spnCol, hvgSettings[0], 4);
      spnCol.SetValueProp(1, 50, 1);
      spnCol.SetValue(1);
  InitControl(lblDir, hvgSettings[2], 0, _M("Direction:"));

#define _EXPAND(X, Y) &hvgSettings[3], hvgSettings[3][0].x1+X, hvgSettings[3][0].y1+Y
  InitControl(idsDir, _EXPAND(92, 15), 152+6, 138+6, picPlaneDirection_xpm);
  InitControl(rdoDirs[0], _EXPAND(175,   0), "( 0, 1, 0)", true);
  InitControl(rdoDirs[1], _EXPAND(165, 160), "( 0,-1, 0)");
  InitControl(rdoDirs[2], _EXPAND(252,  80), "( 1, 0, 0)");
  InitControl(rdoDirs[3], _EXPAND( 88,  80), "(-1, 0, 0)", true);
  InitControl(rdoDirs[4], _EXPAND(131, 160), "( 0, 0,-1)", true);
  InitControl(rdoDirs[5], _EXPAND(210,   0), "( 0, 0, 1)");
  for(size_t i = 0; i < elemsize(rdoDirs); ++i) Connect(rdoDirs[i], CDP_SLOT(CPage_PNS, _onRdoDirs));
  GroupRadioButtons(&rdoDirs[0], &rdoDirs[1], &rdoDirs[2],
                    &rdoDirs[3], &rdoDirs[4], &rdoDirs[5], NULL);
  rdoDirs[0].SetValue(true);
#undef _EXPAND
  InitControl(lblDirInfo, hvgSettings[5], CGLayout::WHOLE_AREA,
              _M(">> As if it is the top side of a 3D cube"));
}

void CPage_PNS::HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str)
{
  UNUSED_PARAM(str);

  if(keySymbol==VKS_ENTER || keySymbol==VKS_KP_ENTER)
    _onBtnOK(NULL);
  else if(keySymbol == VKS_ESC)
    _onBtnCancel(NULL);
}
