// ############################################################################################
//  roomeditt-Main.cc
//  Implementation of roomedit's utility application
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// roomedit's headers
#include "roomedit-RoomEdit.h"


//
// Main program
//
int main(int argc, char *argv[])
{
  // Minimum width and height
  const int minWidth  = 560;
  const int minHeight = 400;

  // Default parameters
  int             width  = 99999;
  int             height = 99999;
  bool            lr3d   = false;
  CRoomEdit::Mode mode   = CRoomEdit::ModeNormal;
  string          file   = "";

  // Parse parameters
  for(int i = 1; i < argc; ++i) {
    const size_t argl = strlen(argv[i]);

    if((argl > 8) && (strncmp(argv[i], "--width=", 8) == 0))
      width = atoi(argv[i] + 8);
    else if((argl > 9) && (strncmp(argv[i], "--height=", 9) == 0))
      height = atoi(argv[i] + 9);
    else if(strcmp(argv[i], "--always-lowres3d") == 0)
      lr3d = true;
    else if((argl > 6) && (strncmp(argv[i], "--new=", 6) == 0)) {
      file = argv[i] + 6;
      mode = CRoomEdit::ModeNew;
    }
    else if((argl > 7) && (strncmp(argv[i], "--open=", 7) == 0)) {
      file = argv[i] + 7;
      mode = CRoomEdit::ModeOpen;
    }
    else {
      VPrint("\n-------------------------------------------");
      VPrint("\nVortexGE Room Editor %s", CRoomEdit::TEXT_VERSION.c_str());
      VPrint("\nCopyright (C) %s Aloysius Indrayanto", CRoomEdit::TEXT_YEAR.c_str());
      VPrint("\n-------------------------------------------");
      VPrint("\nThis program is part of VortexGE and is licensed under the GNU LGPL.\n");
      VPrint("\n");
      VPrint("\nINVALID OPTION: %s\n", argv[i]);
      VPrint("\n");
      VPrint("\nSupported options:\n");
      VPrint("\n  --width=WINDOW_WIDTH");
      VPrint("\n      set the width of the window (minimum %d pixels); the default is to fill the whole desktop width.", minWidth);
      VPrint("\n");
      VPrint("\n  --height=WINDOW_HEIGHT");
      VPrint("\n      set the height of the window (minimum %d pixels); the default is to fill the whole desktop height.", minHeight);
      VPrint("\n");
      VPrint("\n  --always-lowres3d");
      VPrint("\n      always use low resolution when rendering.");
      VPrint("\n");
      VPrint("\n  --new=FILE_NAME");
      VPrint("\n      create a new file with the given name.");
      VPrint("\n");
      VPrint("\n  --open=FILE_NAME");
      VPrint("\n      open the given file name.");
      VPrint("\n\n");
      return(1);
    }
  }

  // Execute the application
  CRoomEdit csMyApp(max(minWidth, width), max(minHeight, height), lr3d, mode, file);
  return(csMyApp.DoMainLoop());
}
